@@ -31,14 +31,21 @@
 #include <klib/text.h>
 #include <klib/printf.h>
 #include <klib/log.h>
+
+#include <kdb/kdb-priv.h> /* KDBManagerGetVFSManager */
 #include <kdb/manager.h>
 #include <kdb/meta.h>
+
 #include <kfg/config.h>
 #include <insdc/insdc.h>
-#include <vdb/database.h>
-#include <vdb/vdb-priv.h>
 #include <vdb/cursor.h>
+#include <vdb/database.h>
+#include <vdb/table.h> /* VDBManagerOpenTableReadVPath */
 #include <vdb/vdb-priv.h>
+
+#include <vfs/manager.h> /* VFSManagerRelease */
+#include <vfs/path-priv.h> /* VPathSetAccOfParentDb */
+
 #include <align/refseq-mgr.h>
 #include <sysalloc.h>
 
@@ -94,6 +101,7 @@ struct RefSeq {
     RefSeqMgr const *mgr;
     struct RefSeq *newer;
     struct RefSeq *older;
+    const String * accOfParentDb;
     union {
         struct RefSeq_RefSeq refSeq;
         struct RefSeq_WGS wgs;
@@ -236,11 +244,11 @@ static RefSeq *RefSeq_RefSeq_init(RefSeq *const super,
                                    char const name[])
 {
     struct RefSeq_RefSeq *const self = &super->u.refSeq;
-    
+
     memmove(self->name, name, namelen);
     self->name[namelen] = '\0';
     super->mgr = mgr;
-    
+
     return super;
 }
 
@@ -252,7 +260,7 @@ static RefSeq *RefSeq_WGS_init(RefSeq *const super,
     struct RefSeq_WGS *const self = &super->u.wgs;
     unsigned prefixLen = 0;
     unsigned digits = 0;
-    
+
     while (prefixLen < namelen && digits < 2) {
         int const ch = name[prefixLen];
         if (isdigit(ch))
@@ -296,7 +304,7 @@ static bool RefSeq_WGS_isopen(RefSeq const *const super)
 static void RefSeq_RefSeq_close(RefSeq *const super)
 {
     TableReaderRefSeq const *const reader = super->u.refSeq.reader;
-    
+
     super->u.refSeq.reader = NULL;
     TableReaderRefSeq_Whack(reader);
 }
@@ -304,7 +312,7 @@ static void RefSeq_RefSeq_close(RefSeq *const super)
 static void RefSeq_WGS_close(RefSeq *const super)
 {
     TableReaderWGS const *const reader = super->u.wgs.reader;
-    
+
     super->u.wgs.reader = NULL;
     TableReaderWGS_Whack(reader);
 }
@@ -325,7 +333,7 @@ static rc_t RefSeq_WGS_setRow(RefSeq *const super,
     assert(strncmp(name, self->name, self->prefixLen) == 0);
     for (i = self->prefixLen; i < N; ++i) {
         int const ch = name[i];
-        
+
         if (isdigit(ch))
             row = (row * 10) + (ch - '0');
         else if (ch == '.')
@@ -395,17 +403,17 @@ static int RefSeq_RefSeq_compare(RefSeq const *super,
 {
     char const *const fnd = super->u.refSeq.name;
     unsigned i;
-    
+
     for (i = 0; ; ++i) {
         int const a = i == qlen ? '\0' : qry[i];
         int const b = fnd[i];
-        
+
         if (a < b)
             return -1;
-        
+
         if (b < a)
             return 1;
-        
+
         if (a == 0)
             break;
     }
@@ -437,7 +445,7 @@ static unsigned FindAccession(unsigned const N,
 {
     unsigned f = 0;
     unsigned e = N;
-    
+
     while (f < e) {
         unsigned const m = f + ((e - f) >> 1);
         int const diff = refSeq[m]->vt->compare(refSeq[m], qlen, qry);
@@ -460,10 +468,10 @@ static rc_t get_schema_info(KMetadata const *const meta,
 {
     KMDataNode const *node;
     rc_t rc = KMetadataOpenNodeRead(meta, &node, "schema");
-    
+
     if (rc == 0) {
         size_t sz;
-        
+
         rc = KMDataNodeReadAttr(node, "name", buffer, bsz - 1, &sz);
         KMDataNodeRelease(node);
         if (rc == 0) {
@@ -485,7 +493,7 @@ static rc_t get_tbl_schema_info(VTable const *const tbl,
 {
     KMetadata const *meta;
     rc_t rc = VTableOpenMetadataRead(tbl, &meta);
-    
+
     buffer[0] = '\0';
     if (rc == 0) rc = get_schema_info(meta, bsz, buffer);
     KMetadataRelease(meta);
@@ -497,7 +505,7 @@ static rc_t get_db_schema_info(VDatabase const *db,
 {
     KMetadata const *meta;
     rc_t rc = VDatabaseOpenMetadataRead(db, &meta);
-    
+
     buffer[0] = '\0';
     if (rc == 0) rc = get_schema_info(meta, bsz, buffer);
     KMetadataRelease(meta);
@@ -506,13 +514,28 @@ static rc_t get_db_schema_info(VDatabase const *db,
 
 static rc_t RefSeq_RefSeq_open(RefSeq *const super, RefSeqMgr const *const mgr)
 {
+    rc_t rc = 0;
+
     struct RefSeq_RefSeq *const self = &super->u.refSeq;
-    VTable const *tbl;
-    rc_t rc = VDBManagerOpenTableRead(mgr->vmgr, &tbl, NULL, self->name);
-    
+
+    VTable const *tbl = NULL;
+    const KDBManager * kmgr = NULL;
+    VFSManager * vfs;
+    VPath * aOrig = NULL;
+
+    rc = VDBManagerGetKDBManagerRead(mgr->vmgr, &kmgr);
+    if (rc == 0)
+        rc = KDBManagerGetVFSManager(kmgr, &vfs);
+    if (rc == 0)
+        rc = VFSManagerMakePath(vfs, &aOrig, "%s", self->name);
+    if (rc == 0)
+        VPathSetAccOfParentDb(aOrig, super->accOfParentDb);
+    if (rc == 0)
+        rc = VDBManagerOpenTableReadVPath(mgr->vmgr, &tbl, NULL, aOrig);
+
     if (tbl) {
         char scheme[1024];
-        
+
         get_tbl_schema_info(tbl, sizeof(scheme), scheme);
         if (strcmp(scheme, "NCBI:refseq:tbl:reference") == 0) {
             rc = TableReaderRefSeq_MakeTable(&self->reader, mgr->vmgr, tbl,
@@ -523,6 +546,11 @@ static rc_t RefSeq_RefSeq_open(RefSeq *const super, RefSeqMgr const *const mgr)
         }
         VTableRelease(tbl);
     }
+
+    VPathRelease(aOrig);
+    VFSManagerRelease(vfs);
+    KDBManagerRelease(kmgr);
+
     return rc;
 }
 
@@ -531,10 +559,10 @@ static rc_t RefSeq_RefSeq_odd_open(RefSeq *const super, RefSeqMgr const *const m
     struct RefSeq_RefSeq *const self = &super->u.refSeq;
     VTable const *tbl;
     rc_t rc = VDBManagerOpenTableRead(mgr->vmgr, &tbl, NULL, "ncbi-acc:%s?vdb-ctx=refseq", self->name);
-    
+
     if (tbl) {
         char scheme[1024];
-        
+
         get_tbl_schema_info(tbl, sizeof(scheme), scheme);
         if (strcmp(scheme, "NCBI:refseq:tbl:reference") == 0) {
             rc = TableReaderRefSeq_MakeTable(&self->reader, mgr->vmgr, tbl,
@@ -556,11 +584,11 @@ static rc_t RefSeq_WGS_open(RefSeq *const super, RefSeqMgr const *const mgr)
 
     if (db) {
         char scheme[1024];
-        
+
         get_db_schema_info(db, sizeof(scheme), scheme);
         if (strcmp(scheme, "NCBI:WGS:db:contig") == 0) {
             VTable const *tbl;
-            
+
             rc = VDatabaseOpenTableRead(db, &tbl, "SEQUENCE");
             if (tbl) {
                 rc = TableReaderWGS_MakeTable(&self->reader, mgr->vmgr, tbl,
@@ -578,41 +606,66 @@ static rc_t RefSeq_WGS_open(RefSeq *const super, RefSeqMgr const *const mgr)
 
 static int AccessionType(VDBManager const *const mgr,
                          unsigned const N, char const accession[],
-                         rc_t *const rc)
+                         rc_t *const rc,
+                         const String * accOfParentDb)
 {
     char scheme[1024];
     bool isOdd = false;
 
+    assert(rc);
+
     scheme[0] = '\0';
+
     {
         KMetadata const *meta = NULL;
         {
+            const KDBManager * kmgr = NULL;
+            /* need VFS manager to make a path */
+            VFSManager * vfs;
             VDatabase const *db = NULL;
+            VPath * aOrig = NULL;
+            *rc = VDBManagerGetKDBManagerRead(mgr, &kmgr);
+            if (*rc == 0)
+                *rc = KDBManagerGetVFSManager(kmgr, &vfs);
 
-            *rc = VDBManagerOpenDBRead(mgr, &db, NULL, "%.*s", (int)N, accession);
-            if (db) {
+            if (*rc == 0) /* create VPath from 'accession' */
+                *rc = VFSManagerMakePath(vfs, &aOrig, "%.*s", (int)N, accession);
+            if (*rc == 0)
+                VPathSetAccOfParentDb(aOrig, accOfParentDb);
+
+            if (*rc == 0) /* use created VPath... */
+                *rc = VDBManagerOpenDBReadVPath(mgr, &db, NULL, aOrig);
+            /* ... or 'accession' if we were not able to create VPath
+               (should never happen, really) */
+            else
+                *rc = VDBManagerOpenDBRead(mgr, &db, NULL, "%.*s", (int)N, accession);
+
+            /* 'accession' was resolved to DataBase */
+            if (db != NULL) {
                 *rc = VDatabaseOpenMetadataRead(db, &meta);
                 VDatabaseRelease(db);
             }
-            else {
+            else { /* try to resolve created accession as Table */
                 VTable const *tbl = NULL;
 
-                *rc = VDBManagerOpenTableRead(mgr, &tbl, NULL, "%.*s", (int)N, accession);
-                if (tbl) {
+          /* VDBManagerOpenTableReadVPath calls names service to resolve path */
+                if (aOrig != NULL)
+                    *rc = VDBManagerOpenTableReadVPath(mgr, &tbl, NULL, aOrig);
+                else { /* should never happen, really) */
+                    isOdd = true;
+                    *rc = VDBManagerOpenTableRead(mgr, &tbl, NULL,
+                        "ncbi-acc:%.*s?vdb-ctx=refseq", (int)N, accession);
+                }
+                if (tbl != NULL) {
                     *rc = VTableOpenMetadataRead(tbl, &meta);
                     VTableRelease(tbl);
                 }
-                else {
-                    isOdd = true;
-                    *rc = VDBManagerOpenTableRead(mgr, &tbl, NULL, "ncbi-acc:%.*s?vdb-ctx=refseq", (int)N, accession);
-                    if (tbl) {
-                        *rc = VTableOpenMetadataRead(tbl, &meta);
-                        VTableRelease(tbl);
-                    }
-                }
             }
+            VPathRelease(aOrig);
+            VFSManagerRelease(vfs);
+            KDBManagerRelease(kmgr);
         }
-        if (meta) {
+        if (meta != NULL) {
             KMDataNode const *node = NULL;
 
             *rc = KMetadataOpenNodeRead(meta, &node, "schema");
@@ -633,6 +686,7 @@ static int AccessionType(VDBManager const *const mgr,
             }
         }
     }
+
     if (strcmp(scheme, "NCBI:WGS:db:contig") == 0)
         return refSeqType_WGS;
     if (strcmp(scheme, "NCBI:refseq:tbl:reference") == 0)
@@ -646,7 +700,7 @@ LIB_EXPORT rc_t CC RefSeqMgr_SetCache(RefSeqMgr const *const cself, size_t cache
 {
     if (cself) {
         RefSeqMgr *const self = (RefSeqMgr *)cself;
-        
+
         WhackAllReaders(self);
         assert(self->num_open == 0);
         self->cache = cache;
@@ -660,7 +714,7 @@ LIB_EXPORT rc_t CC RefSeqMgr_Make( const RefSeqMgr** cself, const VDBManager* vm
 {
     rc_t rc = 0;
     RefSeqMgr* obj = NULL;
-    
+
     if ( cself == NULL || vmgr == NULL )
     {
         rc = RC( rcAlign, rcIndex, rcConstructing, rcParam, rcNull );
@@ -702,6 +756,15 @@ LIB_EXPORT rc_t CC RefSeqMgr_Make( const RefSeqMgr** cself, const VDBManager* vm
     return rc;
 }
 
+void RefseqWhack(RefSeq * self) {
+    assert(self);
+
+    StringWhack(self->accOfParentDb);
+    memset(self, 0, sizeof *self);
+
+    free(self);
+}
+
 LIB_EXPORT rc_t CC RefSeqMgr_Release(const RefSeqMgr* cself)
 {
     if( cself != NULL ) {
@@ -710,7 +773,7 @@ LIB_EXPORT rc_t CC RefSeqMgr_Release(const RefSeqMgr* cself)
 
         WhackAllReaders(self);
         for (i = 0; i < self->nRefSeqs; ++i)
-            free(self->refSeq[i]);
+            RefseqWhack(self->refSeq[i]);
         free(self->refSeq);
         VDBManagerRelease(self->vmgr);
         KConfigRelease(self->kfg);
@@ -723,15 +786,18 @@ static rc_t NewRefSeq(RefSeqMgr *const self,
                                 int const type,
                                 unsigned const at,
                                 unsigned const N,
-                                char const accession[])
+                                char const accession[],
+                                const String * accOfParentDb)
 {
+    rc_t rc = 0;
+
     if (self->nRefSeqs + 1 > self->maxRefSeqs) {
         unsigned const maxRefSeqs = (self->maxRefSeqs == 0) ? 32 : (self->maxRefSeqs << 1);
         void *tmp = realloc(self->refSeq, maxRefSeqs * sizeof(self->refSeq[0]));
-        
+
         if (tmp == NULL)
             return RC(rcAlign, rcTable, rcAccessing, rcMemory, rcExhausted);
-        
+
         self->maxRefSeqs = maxRefSeqs;
         self->refSeq = tmp;
     }
@@ -740,7 +806,7 @@ static rc_t NewRefSeq(RefSeqMgr *const self,
     ++self->nRefSeqs;
     {
         RefSeq *rs = NULL;
-    
+
         switch (type) {
         case refSeqType_RefSeq:
             rs = RefSeq_RefSeq_alloc(N);
@@ -756,13 +822,16 @@ static rc_t NewRefSeq(RefSeqMgr *const self,
             break;
         }
         self->refSeq[at] = rs;
-    
+
         if (rs == NULL)
             return RC(rcAlign, rcTable, rcAccessing, rcMemory, rcExhausted);
 
         rs->vt->init(rs, self, N, accession);
+        if (accOfParentDb != NULL)
+            rc = StringCopy(&rs->accOfParentDb, accOfParentDb);
     }
-    return 0;
+
+    return rc;
 }
 
 static rc_t exists(RefSeqMgr *const self, unsigned const N, char const accession[])
@@ -775,18 +844,18 @@ static rc_t exists(RefSeqMgr *const self, unsigned const N, char const accession
     if (matched)
         return 0;
     {
-        int const type = AccessionType(self->vmgr, N, accession, &rc);
+        int const type = AccessionType(self->vmgr, N, accession, &rc, NULL);
         if (type)
             rc = 0;
         else if (rc == 0)
             rc = RC(rcAlign, rcTable, rcAccessing, rcType, rcUnexpected);
         if (rc == 0)
-            rc = NewRefSeq(self, type, at, N, accession);
+            rc = NewRefSeq(self, type, at, N, accession, NULL);
         else {
             ALIGN_CF_DBG("failed to open %.*s", N, accession);
             ALIGN_DBGERR(rc);
         }
-    }    
+    }
     return rc;
 }
 
@@ -816,22 +885,22 @@ static void MakeNewest(RefSeqMgr *const self, RefSeq *const rs)
 }
 
 static rc_t GetSeq(RefSeqMgr *const self, RefSeq **result,
-                               unsigned const N, char const accession[])
+            unsigned const N, char const accession[], const String * accOfParentDb)
 {
     rc_t rc = 0;
     bool matched = false;
     unsigned const at = FindAccession(self->nRefSeqs,
                                       (RefSeq const **)self->refSeq,
                                       N, accession, &matched);
-    
+
     if (!matched) {
-        int const type = AccessionType(self->vmgr, N, accession, &rc);
+        int const type = AccessionType(self->vmgr, N, accession, &rc, accOfParentDb);
         if (type)
             rc = 0;
         else if (rc == 0)
             rc = RC(rcAlign, rcTable, rcAccessing, rcType, rcUnexpected);
         if (rc == 0)
-            rc = NewRefSeq(self, type, at, N, accession);
+            rc = NewRefSeq(self, type, at, N, accession, accOfParentDb);
         else {
             ALIGN_CF_DBG("failed to open %.*s", N, accession);
             ALIGN_DBGERR(rc);
@@ -857,24 +926,38 @@ LIB_EXPORT rc_t RefSeqMgr_Exists(const RefSeqMgr* cself, const char* accession,
     }
 }
 
-LIB_EXPORT rc_t CC RefSeqMgr_Read(const RefSeqMgr* cself, const char* seq_id, uint32_t seq_id_sz,
-                                  INSDC_coord_zero offset, INSDC_coord_len len,
-                                  uint8_t* buffer, INSDC_coord_len* written)
+static rc_t RefSeqMgr_GetSeqForDb(RefSeqMgr const *const cmgr,
+    RefSeq const **result, char const *seq_id, uint32_t seq_id_sz,
+    const String * accOfParentDb);
+
+LIB_EXPORT rc_t CC RefSeqMgr_ReadForDb(const RefSeqMgr* cself, const char* seq_id,
+    uint32_t seq_id_sz, INSDC_coord_zero offset, INSDC_coord_len len,
+    uint8_t* buffer, INSDC_coord_len* written, const String * accOfParentDb)
 {
     RefSeq const *obj;
     rc_t rc = 0;
 
-    if( (rc = RefSeqMgr_GetSeq(cself, &obj, seq_id, seq_id_sz)) == 0 ) {
+    if( (rc = RefSeqMgr_GetSeqForDb(cself, &obj, seq_id, seq_id_sz, accOfParentDb))
+        == 0 )
+    {
         rc = RefSeq_Read(obj, offset, len, buffer, written);
         RefSeq_Release(obj);
     }
     return rc;
 }
 
+LIB_EXPORT rc_t CC RefSeqMgr_Read(const RefSeqMgr* cself, const char* seq_id, uint32_t seq_id_sz,
+                                  INSDC_coord_zero offset, INSDC_coord_len len,
+                                  uint8_t* buffer, INSDC_coord_len* written)
+{
+    return RefSeqMgr_ReadForDb(cself, seq_id,
+        seq_id_sz, offset, len, buffer, written, NULL);
+}
+
 static void WhackAllReaders(RefSeqMgr *const self)
 {
     unsigned i;
-    
+
     for (i = 0; i < self->nRefSeqs; ++i) {
         RefSeq *const rs = self->refSeq[i];
 
@@ -921,12 +1004,13 @@ static rc_t GetReader(RefSeqMgr *const self, RefSeq *const obj)
 static rc_t GetSeqInternal(RefSeqMgr *const self,
                               RefSeq const **const result,
                               unsigned const seq_id_sz,
-                              char const seq_id[])
+                              char const seq_id[],
+                              const String * accOfParentDb)
 {
     RefSeq *obj = NULL;
-    
+
     if (self->mru == NULL || self->mru->vt->compare(self->mru, seq_id_sz, seq_id) != 0) {
-        rc_t const rc = GetSeq(self, &obj, seq_id_sz, seq_id);
+        rc_t const rc = GetSeq(self, &obj, seq_id_sz, seq_id, accOfParentDb);
         if (rc)
             return rc;
     }
@@ -942,17 +1026,19 @@ static rc_t GetSeqInternal(RefSeqMgr *const self,
     return 0;
 }
 
-LIB_EXPORT rc_t CC RefSeqMgr_GetSeq(RefSeqMgr const *const cmgr,
+static rc_t RefSeqMgr_GetSeqForDb  (RefSeqMgr const *const cmgr,
                                     RefSeq const **result,
                                     char const *seq_id,
-                                    uint32_t seq_id_sz)
+                                    uint32_t seq_id_sz,
+                                    const String * accOfParentDb)
 {
     rc_t rc;
 
     if (cmgr == NULL || result == NULL || seq_id == NULL)
         rc = RC(rcAlign, rcIndex, rcConstructing, rcParam, rcNull);
     else
-        rc = GetSeqInternal((RefSeqMgr *)cmgr, result, seq_id_sz, seq_id);
+        rc = GetSeqInternal((RefSeqMgr *)cmgr, result, seq_id_sz, seq_id,
+            accOfParentDb);
 
     if (rc)
         ALIGN_DBGERRP("SEQ_ID: '%.*s'", rc, seq_id_sz, seq_id);
@@ -960,6 +1046,14 @@ LIB_EXPORT rc_t CC RefSeqMgr_GetSeq(RefSeqMgr const *const cmgr,
     return rc;
 }
 
+LIB_EXPORT rc_t CC RefSeqMgr_GetSeq(RefSeqMgr const *const cmgr,
+                                    RefSeq const **result,
+                                    char const *seq_id,
+                                    uint32_t seq_id_sz)
+{
+    return RefSeqMgr_GetSeqForDb(cmgr, result, seq_id, seq_id_sz, NULL);
+}
+
 LIB_EXPORT rc_t CC RefSeq_Read(const RefSeq* cself, INSDC_coord_zero offset, INSDC_coord_len len,
                                uint8_t* buffer, INSDC_coord_len* written)
 {
@@ -970,7 +1064,7 @@ LIB_EXPORT rc_t CC RefSeq_Read(const RefSeq* cself, INSDC_coord_zero offset, INS
     else {
         RefSeq *const self = (RefSeq *)cself;
         RefSeqMgr *const mgr = (RefSeqMgr *)self->mgr;
-        
+
         rc = GetReader(mgr, self);
         if (rc == 0)
             rc = self->vt->read(self, offset, len, buffer, written);
@@ -987,7 +1081,7 @@ LIB_EXPORT rc_t CC RefSeq_Circular(const RefSeq* cself, bool* circular)
         rc = RC(rcAlign, rcFile, rcReading, rcParam, rcNull);
     else {
         RefSeq const *const self = (RefSeq *)cself;
-        
+
         rc = self->vt->circular(self, circular);
     }
     ALIGN_DBGERR(rc);
@@ -1002,7 +1096,7 @@ LIB_EXPORT rc_t CC RefSeq_SeqLength(const RefSeq* cself, INSDC_coord_len* len)
         rc = RC(rcAlign, rcFile, rcReading, rcParam, rcNull);
     else {
         RefSeq const *const self = (RefSeq *)cself;
-        
+
         rc = self->vt->length(self, len);
     }
     ALIGN_DBGERR(rc);
@@ -1017,7 +1111,7 @@ LIB_EXPORT rc_t CC RefSeq_MD5(const RefSeq* cself, const uint8_t** md5)
         rc = RC(rcAlign, rcFile, rcReading, rcParam, rcNull);
     else {
         RefSeq const *const self = (RefSeq *)cself;
-        
+
         rc = self->vt->checksum(self, md5);
     }
     ALIGN_DBGERR(rc);
@@ -1027,12 +1121,12 @@ LIB_EXPORT rc_t CC RefSeq_MD5(const RefSeq* cself, const uint8_t** md5)
 LIB_EXPORT rc_t CC RefSeq_Name(const RefSeq* cself, const char** name)
 {
     rc_t rc = 0;
-    
+
     if (cself == NULL)
         rc = RC(rcAlign, rcFile, rcReading, rcParam, rcNull);
     else {
         RefSeq const *const self = (RefSeq *)cself;
-        
+
         *name = self->vt->name(self);
     }
     ALIGN_DBGERR(rc);
@@ -1060,7 +1154,7 @@ rc_t RefSeqMgr_ConfigValue ( const KConfig *kfg, const char *node_path, char *va
             else
                 value [ num_read ] = 0;
         }
-        
+
         KConfigNodeRelease ( node );
     }
     return rc;
@@ -1072,12 +1166,12 @@ rc_t RefSeqMgr_KfgReadRepositories(const KConfig* kfg, char* paths, size_t paths
     /* servers are children of refseq/repository, e.g.:             /refseq/repository/main="..." */
     /* volumes are in refseq/repository/<serverName>/volumes, e.g.: /refseq/repository/main/volumes="..." */
     /* all server/volume combinations are returned in paths separated by ':' */
-    
+
     rc_t rc = 0;
     const KConfigNode *node;
 #define KFG_PATH "/refseq/repository/"
     paths[0] = 0;
-    
+
     rc = KConfigOpenNodeRead ( kfg, & node, KFG_PATH );
     if ( rc == 0 )
     {
@@ -1100,7 +1194,7 @@ rc_t RefSeqMgr_KfgReadRepositories(const KConfig* kfg, char* paths, size_t paths
                         char server[ BufSize ];
                         char buf[ BufSize ];
                         size_t num_writ;
-                        
+
                         rc = string_printf(buf, BufSize, &num_writ, KFG_PATH "%s", name);
                         if (rc == 0)
                         {
@@ -1116,7 +1210,7 @@ rc_t RefSeqMgr_KfgReadRepositories(const KConfig* kfg, char* paths, size_t paths
                                     {   /* create a server/volume pair for every combination, append to paths, ':' - separate */
                                         char *vol_rem = volumes;
                                         char *vol_sep;
-                                        
+
                                         do {
                                             char const *volume = vol_rem;
                                             vol_sep = string_chr(volume, string_size(volume), ':');
@@ -1146,7 +1240,7 @@ rc_t RefSeqMgr_KfgReadRepositories(const KConfig* kfg, char* paths, size_t paths
             }
             KNamelistRelease ( children );
         }
-        
+
         KConfigNodeRelease ( node );
     }
     if (GetRCState(rc) == rcNotFound)
@@ -1162,7 +1256,7 @@ rc_t RefSeqMgr_KfgReadStr(const KConfig* kfg, const char* path, char* value, siz
 {
     rc_t rc = 0;
     const KConfigNode *node;
-    
+
     if ( (rc = KConfigOpenNodeRead(kfg, &node, "%s", path)) == 0 ) {
         size_t num_read, remaining;
         if( (rc = KConfigNodeRead(node, 0, value, value_sz - 1, &num_read, &remaining)) == 0 ) {
@@ -1185,7 +1279,7 @@ rc_t RefSeqMgr_ForEachVolume(const RefSeqMgr* cself, RefSeqMgr_ForEachVolume_cal
     rc_t rc = 0;
     char servers[4096];
     char volumes[4096];
-    
+
     if( cself == NULL || cb == NULL ) {
         rc = RC(rcAlign, rcType, rcConstructing, rcParam, rcNull);
     } else if( cb(".", NULL, data) ) {
@@ -1199,7 +1293,7 @@ rc_t RefSeqMgr_ForEachVolume(const RefSeqMgr* cself, RefSeqMgr_ForEachVolume_cal
             char *srv_rem = servers;
             do {
                 char const* server = srv_rem;
-                
+
                 srv_sep = strchr(server, ':');
                 if(srv_sep) {
                     srv_rem = srv_sep + 1;
@@ -1221,7 +1315,7 @@ rc_t RefSeqMgr_ForEachVolume(const RefSeqMgr* cself, RefSeqMgr_ForEachVolume_cal
                 char *srv_rem = servers;
                 do {
                     char const* server = srv_rem;
-                    
+
                     srv_sep = strchr(server, ':');
                     if(srv_sep) {
                         srv_rem = srv_sep + 1;
@@ -1248,7 +1342,7 @@ rc_t RefSeqMgr_ForEachVolume(const RefSeqMgr* cself, RefSeqMgr_ForEachVolume_cal
                     char const *server = srv_rem;
                     char *vol_rem = vol;
                     char *vol_sep;
-                    
+
                     string_copy ( vol, sizeof vol, volumes, string_size( volumes ) );
                     srv_sep = strchr(server, ':');
                     if(srv_sep) {
@@ -1257,7 +1351,7 @@ rc_t RefSeqMgr_ForEachVolume(const RefSeqMgr* cself, RefSeqMgr_ForEachVolume_cal
                     }
                     do {
                         char const *volume = vol_rem;
-                        
+
                         vol_sep = strchr(volume, ':');
                         if(vol_sep) {
                             vol_rem = vol_sep + 1;
