@@ -27,6 +27,10 @@
 #ifndef _h_kfg_priv_
 #define _h_kfg_priv_
 
-extern void add_aws_nodes ( KConfig *self );
+struct KConfig;
+
+extern void add_aws_nodes ( struct KConfig *self );
+
+const char * KConfigGetNgcFile(void);
 
 #endif /* _h_kfg_priv_ */
