@@ -1829,6 +1829,116 @@ rc_t KSysDirCreateAlias_v1 ( KSysDir_v1 * self,
     return rc;
 }
 
+
+/* CreateLink ( v1.5 )
+ *  creates a new link (also known as a hard link).
+ *
+ *  "access" [ IN ] - standard Unix directory access mode
+ *  used when "mode" has kcmParents set and alias path does
+ *  not exist.
+ *
+ *  "mode" [ IN ] - a creation mode ( see explanation above ).
+ *
+ *  "oldpath" [ IN ] - NUL terminated string in directory-native
+ *  character set denoting existing object. THE PATH IS GIVEN RELATIVE
+ *  TO DIRECTORY ( "self" ), NOT LINK ( "newpath" )!
+ *
+ *  "newpath" [ IN ] - NUL terminated string in directory-native
+ *  character set denoting a new link.
+ */
+static
+rc_t KSysDirCreateLink_v1 ( KSysDir_v1 * self,
+    uint32_t access, KCreateMode mode,
+    const char *oldpath, const char *newpath )
+{
+    /* create full path to link */
+    char flink [ PATH_MAX ] = "";
+    rc_t rc = KSysDirMakePath_v1 ( self, rcCreating, true,
+        flink, sizeof flink, newpath, NULL );
+    if ( rc == 0 )
+    {
+        /* the full path to oldpath RELATIVE TO self */
+        char fold [ PATH_MAX ] = "";
+        rc = KSysDirMakePath_v1 ( self, rcCreating, true,
+            fold, sizeof fold, oldpath, NULL );
+        if ( rc == 0 )
+        {
+            /* if "self" is chroot'd, "fold" must be made relative */
+            if ( self -> root != 0 )
+            {
+                /* take path to newpath as root.
+                   generate a path RELATIVE TO newpath */
+                rc = KSysDirRelativePath_v1 ( self, rcCreating, flink,
+                    fold, sizeof fold /*strlen ( fold )*/ );
+                if ( rc != 0 )
+                    return rc;
+            }
+
+            if ( link ( fold, flink ) == 0 )
+                return 0;
+
+            switch ( errno )
+            {
+            case EMLINK:
+                /* The number of links to the file named by oldpath
+                   would exceed {LINK_MAX} */
+                return RC ( rcFS, rcDirectory, rcCreating,
+                    rcFile, rcExcessive );
+
+            case EXDEV:
+                /* The link named by newpath and the file named by oldpath are
+                   on different file systems and the implementation does not
+                   support links between file systems. */
+                return RC ( rcFS, rcDirectory, rcCreating,
+                    rcPath, rcIncorrect );
+
+            case EEXIST:
+                /* newpath already exists */
+                return RC ( rcFS, rcDirectory, rcCreating, rcPath, rcExists );
+
+            case ENOENT:
+                /* a part of the newpath path doesn't exist */
+                if ( ( mode & kcmParents ) != 0 )
+                {
+                    KSysDirCreateParents_v1 ( self, flink, access, true );
+                    break;
+                }
+                return RC ( rcFS, rcDirectory, rcCreating, rcPath, rcNotFound );
+
+            case EPERM:
+            case EACCES:
+            case EROFS:
+                return RC ( rcFS, rcDirectory, rcCreating,
+                    rcDirectory, rcUnauthorized );
+            case ENAMETOOLONG:
+                return RC ( rcFS, rcDirectory, rcCreating,
+                    rcPath, rcExcessive );
+            case ENOTDIR:
+            case ELOOP:
+                return RC ( rcFS, rcDirectory, rcCreating, rcPath, rcInvalid );
+            case ENOSPC:
+                return RC ( rcFS, rcDirectory, rcCreating,
+                    rcStorage, rcExhausted );
+            default:
+                return RC ( rcFS, rcDirectory, rcCreating, rcNoObj, rcUnknown );
+            }
+
+            /* try with missing directories created */
+            if ( link ( fold, flink ) != 0 ) switch ( errno )
+            {
+            case ENOENT:
+                return RC ( rcFS, rcDirectory, rcCreating, rcPath, rcNotFound );
+            default:
+                return RC ( rcFS, rcDirectory, rcCreating, rcNoObj, rcUnknown );
+            }
+
+            assert ( rc == 0 );
+        }
+    }
+    return rc;
+}
+
+
 /* KSysDirOpenFileRead
  *  opens an existing file with read-only access
  *
@@ -2361,8 +2471,8 @@ rc_t CC KSysDirFileContiguous_v1 ( const KSysDir_v1 *self,
 
 static KDirectory_vt_v1 vtKSysDir =
 {
-    /* version 1.4 */
-    1, 4,
+    /* version 1.5 */
+    1, 5,
 
     /* start minor version 0*/
     KSysDirDestroy_v1,
@@ -2412,8 +2522,12 @@ static KDirectory_vt_v1 vtKSysDir =
     /* end minor version 3 */
 
     /* start minor version 4 */
-    KSysDirOpenFileWrite_v1
+    KSysDirOpenFileWrite_v1,
     /* end minor version 4 */
+
+    /* start minor version 5 */
+    KSysDirCreateLink_v1,
+    /* end minor version 5 */
 };
 
 /* KSysDirInit
