@@ -199,7 +199,6 @@ rc_t KHttpFileMakeRequest ( const KHttpFile *cself, uint64_t pos, size_t req_siz
                 KClientHttpRequestSetCloudParams(req, cself -> need_env_token, cself -> payRequired);
                 if ( cself -> need_env_token && ! cself -> url_is_temporary)
                 {
-                    KClientHttpRequestSetCloudParams(req, true, cself -> payRequired);
                     KClientHttpRequestAttachEnvironmentToken ( req );
                     /* TBD - there should be a version of POST that takes a timeout */
                     rc = KClientHttpRequestPOST ( req, rslt );
@@ -849,9 +848,16 @@ rc_t CC KHttpFileRead ( const KHttpFile *self, uint64_t pos,
      void *buffer, size_t bsize, size_t *num_read )
 {
     struct timeout_t tm;
-    TimeoutInit ( & tm, self -> kns -> http_read_timeout );
+    timeout_t * ptm = NULL;
 
-    return KHttpFileTimedRead ( self, pos, buffer, bsize, num_read, & tm );
+    assert(self && self->kns);
+
+    if (self -> kns -> http_read_timeout >= 0) {
+        TimeoutInit ( & tm, self -> kns -> http_read_timeout );
+        ptm = &tm;
+    }
+
+    return KHttpFileTimedRead ( self, pos, buffer, bsize, num_read, ptm );
 }
 
 static
@@ -1216,9 +1222,17 @@ rc_t CC KHttpFileReadChunked ( const KHttpFile * self, uint64_t pos,
     KChunkReader * chunks, size_t bytes, size_t * num_read )
 {
     struct timeout_t tm;
-    TimeoutInit ( & tm, self -> kns -> http_read_timeout );
+    timeout_t * ptm = NULL;
 
-    return KHttpFileTimedReadChunked ( self, pos, chunks, bytes, num_read, & tm );
+    assert(self && self->kns);
+
+    if (self -> kns -> http_read_timeout >= 0) {
+        TimeoutInit ( & tm, self -> kns -> http_read_timeout );
+        ptm = &tm;
+    }
+
+    return KHttpFileTimedReadChunked ( self, pos, chunks, bytes, num_read,
+        ptm );
 }
 #endif /* SUPPORT_CHUNKED_READ */
 
@@ -1331,11 +1345,11 @@ static rc_t KNSManagerVMakeHttpFileIntUnstableImpl( const KNSManager *self,
                     {
                         KClientHttpResult *rslt;
 
+                        KClientHttpRequestSetCloudParams ( req, need_env_token, payRequired );
                         if ( need_env_token )
                         {
                             KClientHttpRequestAttachEnvironmentToken ( req );
                         }
-                        KClientHttpRequestSetCloudParams ( req, need_env_token, payRequired );
 
                         rc = KClientHttpRequestHEAD ( req, & rslt );
                         if ( rc == 0 && rslt -> expiration != NULL )
@@ -1420,6 +1434,14 @@ static rc_t KNSManagerVMakeHttpFileIntUnstableImpl( const KNSManager *self,
                                         f -> need_env_token = need_env_token;
                                         f -> payRequired = payRequired;
 
+                               /* readWaitMillis and totalReadWaitMillis
+                                  ARE NEEDED BY
+                                  stable-http-file.c : HttpFileGetReadTimeouts()
+                                */
+                                        f -> totalReadWaitMillis =
+                                            f -> readWaitMillis
+                                            = self -> http_read_timeout;
+
                                         * file = & f -> dad;
                                         return 0;
                                     }
@@ -1438,7 +1460,7 @@ static rc_t KNSManagerVMakeHttpFileIntUnstableImpl( const KNSManager *self,
                                             String ext;
                                             StringInit ( & ext,
                                                     base + buf -> elem_count - vdbcache . size - 1,
-                                                    vdbcache . size, vdbcache . size );
+                                                    vdbcache . size, vdbcache . len );
                                             if ( ext . addr [ ext . size ] == '\0' &&
                                                 StringEqual ( & vdbcache, & ext ) )
                                             {
@@ -1448,7 +1470,7 @@ static rc_t KNSManagerVMakeHttpFileIntUnstableImpl( const KNSManager *self,
                                                 size_t size = query - base;
                                                 StringInit ( & ext,
                                                     base + size - vdbcache . size,
-                                                    vdbcache . size, vdbcache . size );
+                                                    vdbcache . size, vdbcache . len );
                                                 if ( ext . addr [ ext . size ] == '?' &&
                                                     StringEqual ( & vdbcache, & ext ) )
                                                 {
@@ -1509,3 +1531,7 @@ rc_t KNSManagerVMakeHttpFileIntUnstable(const KNSManager *self,
     return KNSManagerVMakeHttpFileIntUnstableImpl(self, file, conn, vers,
         reliable, need_env_token, payRequired, NULL, url, args);
 }
+
+bool KUnstableFileIsKHttpFile(const KFile * self) {
+    return self != NULL && &self->vt->v1 == &vtKHttpFile;
+}
