@@ -59,14 +59,27 @@
 #include <kns/endpoint.h>
 #endif
 
+/* timeout on Http Read */
 #ifndef MAX_HTTP_READ_LIMIT
 #define MAX_HTTP_READ_LIMIT ( 5 * 60 * 1000 ) /* 5 minutes */
 #endif
 
+/* timeout on Http Write */
 #ifndef MAX_HTTP_WRITE_LIMIT
 #define MAX_HTTP_WRITE_LIMIT ( 15 * 1000 )
 #endif
 
+/* timeout on Http Read loop */
+#ifndef MAX_HTTP_TOTAL_READ_LIMIT
+#define MAX_HTTP_TOTAL_READ_LIMIT ( 10 * 60 * 1000 ) /* 10 minutes */
+#endif
+
+/* timeout on Http Connect loop */
+#ifndef MAX_HTTP_TOTAL_CONNECT_LIMIT
+#define MAX_HTTP_TOTAL_CONNECT_LIMIT ( 10 * 60 * 1000 ) /* 10 minutes */
+#endif
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -274,6 +287,8 @@ rc_t KNSManagerVMakeHttpFileIntUnstable(const struct KNSManager *self,
     const struct KFile **file, struct KStream *conn, ver_t vers, bool reliable,
     bool need_env_token, bool payRequired, const char *url, va_list args);
 
+bool KUnstableFileIsKHttpFile(const struct KFile * self);
+
 #ifdef __cplusplus
 }
 #endif
