@@ -27,6 +27,7 @@
 
 #include <kns/extern.h>
 
+#include <kns/http-priv.h> /* HttpFileGetReadTimeouts */
 #include <kns/stream.h> /* KStreamRelease */
 
 #include <klib/debug.h> /* KStsLevel */
@@ -34,6 +35,7 @@
 #include <klib/printf.h> /* KDataBufferVPrintf */
 #include <klib/rc.h> /* RC */
 #include <klib/status.h> /* KStsLevelGet */
+#include <klib/strings.h> /* ENV_VAR_LOG_HTTP_RETRY */
 #include <klib/time.h> /* KSleep */
 
 #include <strtol.h> /* strtou64 */
@@ -51,6 +53,15 @@ static
 void RetrierReset(const KStableHttpFile * cself, const char * func)
 {
     KStableHttpFile * self = (KStableHttpFile *)cself;
+
+    static int logLevel = -1;
+    if (logLevel == -1) {
+        logLevel = 0;
+        const char * e = getenv(ENV_VAR_LOG_HTTP_RETRY);
+        if (e != NULL)
+            logLevel = atoi(e);
+    }
+
     self->live = true;
 
     if (self->_failed) {
@@ -60,31 +71,72 @@ void RetrierReset(const KStableHttpFile * cself, const char * func)
         self->_state = eRSJustRetry;
         self->_sleepTO = 0;
 
-        if (KStsLevelGet() >= 1)
+        if (logLevel > 1 || KStsLevelGet() > 0)
             PLOGERR(klogErr, (klogErr, 0, "$(f) success", "f=%s", func));
     }
 }
 
 /* reopen the underlying file */
 static
-rc_t RetrierReopenRemote(KStableHttpFile * self)
+rc_t RetrierReopenRemote(KStableHttpFile * self, bool neverBefore)
 {
     rc_t rc = 0;
 
     uint32_t timeout = 1;
 
+    KTime_t started = KTimeStamp();
+    bool first = true;
+    int i = 1;
+
+    int logLevel = -1;
+    if (logLevel == -1) {
+        logLevel = 0;
+        const char * e = getenv(ENV_VAR_LOG_HTTP_RETRY);
+        if (e != NULL)
+            logLevel = atoi(e);
+    }
+
     KFileRelease(self->file);
     self->file = NULL;
 
-    while (timeout < 10 * 60 /* 10 minutes */) {
+    while ( first 
+        || ( self->totalConnectWaitMillis < 0 )
+        || ( KTimeStamp() - started
+            < self->totalConnectWaitMillis / 1000 ) )
+    {
         rc = KNSManagerVMakeHttpFileIntUnstableFromBuffer(self->mgr,
             &self->file, self->conn, self->vers, self->reliable,
             self->need_env_token, self->payRequired, self->url, &self->buf);
-        if (rc == 0)
+        if (rc == 0) {
+            if (neverBefore && !first && logLevel > 0)
+                PLOGERR(klogErr, (klogErr, rc,
+                    "OpenRemoteFile success: attempt $(n)", "n=%d", i));
             break;
+        }
+        else if (neverBefore) {
+            if (!self->reliable)
+                break;
+            else {
+                if (GetRCObject(rc) ==                 rcConnection ||
+                    GetRCObject(rc) == (enum RCObject)(rcTimeout))
+                {
+                    if (logLevel > 0)
+                        PLOGERR(klogErr, (klogErr, rc, "Cannot OpenRemoteFile: "
+                            "retrying $(n)...", "n=%d", i));
+                }
+                else {
+                    if (logLevel > 0)
+                        LOGERR(klogErr, rc, "Cannot OpenRemoteFile");
+                    break;
+                }
+            }
+        }
 
+        first = false;
         KSleep(timeout);
-        timeout *= 2;
+        ++timeout;
+        if (neverBefore)
+            ++i;
     }
 
     return rc;
@@ -94,20 +146,61 @@ rc_t RetrierReopenRemote(KStableHttpFile * self)
 static
 bool RetrierIncSleepTO(KStableHttpFile * self)
 {
-    if (self->_sleepTO == 0)
-        self->_sleepTO = 1;
-    else
-        self->_sleepTO *= 2;
-
-    if (self->_sleepTO == 16)
-        self->_sleepTO = 15;
-
-    if (self->_sleepTO > 20 * 60 /* 20 minutes */)
+    if (++self->_sleepTO > 20 * 60 /* 20 minutes */)
         return false;
     else
         return true;
 }
 
+
+/* Get read timeout and Maximum TotalWait time for Read calls */
+rc_t HttpFileGetReadTimeouts(const KFile * self, int32_t * millis,
+    int32_t * totalMillis)
+{
+    const KStableHttpFile * obj = (const KStableHttpFile*)self;
+
+    if (millis == NULL || totalMillis == NULL)
+        return RC(rcNS, rcFile, rcAccessing, rcParam, rcNull);
+
+    if (obj == NULL)
+        return RC(rcNS, rcFile, rcAccessing, rcSelf, rcNull);
+
+    *millis = obj->readWaitMillis;
+    *totalMillis = obj->totalReadWaitMillis;
+
+    return 0;
+}
+
+/* SET readWaitMillis and totalReadWaitMillis FOR HttpFileGetReadTimeouts() */
+static
+int32_t CalculateTotalReadWait(const KStableHttpFile * self,
+    int32_t * ms)
+{
+    assert(self && self->mgr && ms);
+
+    *ms = self->mgr->http_read_timeout;
+
+    if (!self->reliable)
+        /* There is no retry for not reliable files.
+           Max wait timeout for regular HTTP file is http_read_timeout */
+        return self->mgr->http_read_timeout;
+    else
+        /* Max wait timeout for reliable files is
+           maxTotalWaitForReliableURLs_ms : see RetrierAgain() */
+        if (self->mgr->http_read_timeout < 0)
+            return self->mgr->http_read_timeout;
+        else if (self->mgr->maxTotalWaitForReliableURLs_ms < 0)
+            return self->mgr->maxTotalWaitForReliableURLs_ms;
+        else if (self->mgr->http_read_timeout >
+            self->mgr->maxTotalWaitForReliableURLs_ms)
+        {
+            return self->mgr->http_read_timeout;
+        }
+        else
+            return self->mgr->maxTotalWaitForReliableURLs_ms;
+}
+
+
 /* The last call to the file was failed: prepare to retry.
  *
  * RetrierAgain returns:
@@ -127,7 +220,11 @@ rc_t RetrierAgain(const KStableHttpFile * cself,
 
     assert(self && self->mgr);
 
-    total = self->mgr->maxTotalWaitForReliableURLs_ms / 1000;
+    if (self->mgr->maxTotalWaitForReliableURLs_ms >= 0)
+        total = self->mgr->maxTotalWaitForReliableURLs_ms / 1000;
+    else
+        total = ~0;
+
     retryFirst = self->mgr->retryFirstRead;
 
     if (total == 0)           /* don't retry when overall retry time == 0 */
@@ -143,7 +240,7 @@ rc_t RetrierAgain(const KStableHttpFile * cself,
         case eRSJustRetry:
             break;
         case eRSReopen:
-            if (RetrierReopenRemote(self) != 0)
+            if (RetrierReopenRemote(self, false) != 0)
                 retry = false;
             break;
         case eRSIncTO:
@@ -162,7 +259,14 @@ rc_t RetrierAgain(const KStableHttpFile * cself,
 
     if (retry) {
         KStsLevel lvl = KStsLevelGet();
-        if (lvl > 0) {
+        static int logLevel = -1;
+        if (logLevel == -1) {
+            logLevel = 0;
+            const char * e = getenv(ENV_VAR_LOG_HTTP_RETRY);
+            if (e != NULL)
+                logLevel = atoi(e);
+        }
+        if (logLevel > 1 || lvl > 0) {
             switch (self->_state) {
             case eRSJustRetry:
                 if (self->live)
@@ -223,7 +327,7 @@ rc_t RetrierAgain(const KStableHttpFile * cself,
 /************************ wrapper KFile implementation ************************/
 
 static
-rc_t CC KHttpFileDestroy(KStableHttpFile *self)
+rc_t CC KStblHttpFileDestroy(KStableHttpFile *self)
 {
     rc_t rc = 0, r2 = 0;
 
@@ -251,7 +355,7 @@ rc_t CC KHttpFileDestroy(KStableHttpFile *self)
 }
 
 static
-struct KSysFile* CC KHttpFileGetSysFile(const KStableHttpFile *self,
+struct KSysFile* CC KStblHttpFileGetSysFile(const KStableHttpFile *self,
     uint64_t *offset)
 {
     *offset = 0;
@@ -259,25 +363,25 @@ struct KSysFile* CC KHttpFileGetSysFile(const KStableHttpFile *self,
 }
 
 static
-rc_t CC KHttpFileRandomAccess(const KStableHttpFile *self)
+rc_t CC KStblHttpFileRandomAccess(const KStableHttpFile *self)
 {
     return KFileRandomAccess(self->file);
 }
 
 static
-rc_t CC KHttpFileSize(const KStableHttpFile *self, uint64_t *size)
+rc_t CC KStblHttpFileSize(const KStableHttpFile *self, uint64_t *size)
 {
     return KFileSize(self->file, size);
 }
 
 static
-rc_t CC KHttpFileSetSize(KStableHttpFile *self, uint64_t size)
+rc_t CC KStblHttpFileSetSize(KStableHttpFile *self, uint64_t size)
 {
     return RC(rcNS, rcFile, rcUpdating, rcFile, rcReadonly);
 }
 
 static
-rc_t CC KHttpFileTimedRead(const KStableHttpFile *self, uint64_t pos,
+rc_t CC KStblHttpFileTimedRead(const KStableHttpFile *self, uint64_t pos,
     void *buffer, size_t bsize, size_t *num_read, struct timeout_t *tm)
 {
     quitting_t quitting = self->quitting;
@@ -302,7 +406,7 @@ rc_t CC KHttpFileTimedRead(const KStableHttpFile *self, uint64_t pos,
 }
 
 static
-rc_t CC KHttpFileRead(const KStableHttpFile *self, uint64_t pos,
+rc_t CC KStblHttpFileRead(const KStableHttpFile *self, uint64_t pos,
     void *buffer, size_t bsize, size_t *num_read)
 {
     quitting_t quitting = self->quitting;
@@ -327,14 +431,14 @@ rc_t CC KHttpFileRead(const KStableHttpFile *self, uint64_t pos,
 }
 
 static
-rc_t CC KHttpFileWrite(KStableHttpFile *self, uint64_t pos,
+rc_t CC KStblHttpFileWrite(KStableHttpFile *self, uint64_t pos,
     const void *buffer, size_t size, size_t *num_writ)
 {
     return RC(rcNS, rcFile, rcUpdating, rcInterface, rcUnsupported);
 }
 
 static
-rc_t CC KHttpFileTimedWrite(KStableHttpFile *self, uint64_t pos,
+rc_t CC KStblHttpFileTimedWrite(KStableHttpFile *self, uint64_t pos,
     const void *buffer, size_t size, size_t *num_writ,
     struct timeout_t *tm)
 {
@@ -342,7 +446,7 @@ rc_t CC KHttpFileTimedWrite(KStableHttpFile *self, uint64_t pos,
 }
 
 static
-uint32_t CC KHttpFileGetType(const KStableHttpFile *self)
+uint32_t CC KStblHttpFileGetType(const KStableHttpFile *self)
 {
     return KFileType(self->file);
 }
@@ -350,8 +454,8 @@ uint32_t CC KHttpFileGetType(const KStableHttpFile *self)
 #if SUPPORT_CHUNKED_READ
 
 static
-rc_t CC KHttpFileTimedReadChunked(const KStableHttpFile * self, uint64_t pos,
-    KChunkReader * chunks, size_t bytes, size_t * num_read,
+rc_t CC KStblHttpFileTimedReadChunked(const KStableHttpFile * self,
+    uint64_t pos, KChunkReader * chunks, size_t bytes, size_t * num_read,
     struct timeout_t * tm)
 {
     quitting_t quitting = self->quitting;
@@ -377,7 +481,7 @@ rc_t CC KHttpFileTimedReadChunked(const KStableHttpFile * self, uint64_t pos,
 }
 
 static
-rc_t CC KHttpFileReadChunked(const KStableHttpFile * self, uint64_t pos,
+rc_t CC KStblHttpFileReadChunked(const KStableHttpFile * self, uint64_t pos,
     KChunkReader * chunks, size_t bytes, size_t * num_read)
 {
     quitting_t quitting = self->quitting;
@@ -412,19 +516,19 @@ static KFile_vt_v1 vtKHttpFile =
     2,
 #endif
 
-    KHttpFileDestroy,
-    KHttpFileGetSysFile,
-    KHttpFileRandomAccess,
-    KHttpFileSize,
-    KHttpFileSetSize,
-    KHttpFileRead,
-    KHttpFileWrite,
-    KHttpFileGetType,
-    KHttpFileTimedRead,
-    KHttpFileTimedWrite
+    KStblHttpFileDestroy,
+    KStblHttpFileGetSysFile,
+    KStblHttpFileRandomAccess,
+    KStblHttpFileSize,
+    KStblHttpFileSetSize,
+    KStblHttpFileRead,
+    KStblHttpFileWrite,
+    KStblHttpFileGetType,
+    KStblHttpFileTimedRead,
+    KStblHttpFileTimedWrite
 #if SUPPORT_CHUNKED_READ
-    , KHttpFileReadChunked
-    , KHttpFileTimedReadChunked
+    , KStblHttpFileReadChunked
+    , KStblHttpFileTimedReadChunked
 #endif
 };
 
@@ -460,6 +564,12 @@ rc_t KHttpFileMake(KStableHttpFile ** self,
     return rc;
 }
 
+enum {
+    eUninitialized,
+    eNotSet,
+    eUnreliable,
+    eReliable,
+};
 
 /********************************* constructor ********************************/
 
@@ -478,8 +588,10 @@ rc_t KNSManagerVMakeHttpFileInt(const KNSManager *self,
     if (file == NULL)
         rc = RC(rcNS, rcFile, rcConstructing, rcParam, rcNull);
     else {
+        *file = NULL;
+
         if (self == NULL)
-            rc = RC(rcNS, rcNoTarg, rcConstructing, rcParam, rcNull);
+            rc = RC(rcNS, rcFile, rcConstructing, rcParam, rcNull);
         else if (url == NULL)
             rc = RC(rcNS, rcFile, rcConstructing, rcPath, rcNull);
         else if (url[0] == 0)
@@ -489,33 +601,56 @@ rc_t KNSManagerVMakeHttpFileInt(const KNSManager *self,
             rc = KHttpFileMake(&f, url, args);
 
             if (rc == 0) {
-                rc = KNSManagerVMakeHttpFileIntUnstableFromBuffer(self,
-                    &f->file, conn, vers, reliable, need_env_token, payRequired,
-                    url, &f->buf);
+                rc = KNSManagerAddRef(self);
+                if (rc == 0)
+                    f->mgr = self;
 
+                if (rc == 0)
+                    rc = KStreamAddRef(conn);
                 if (rc == 0) {
-                    rc = KNSManagerAddRef(self);
-
-                    if (rc == 0) {
-                        f->mgr = self;
-
-                        rc = KStreamAddRef(conn);
-                        if (rc == 0) {
-                            f->conn = conn;
-
-                            f->vers = vers;
-                            f->reliable = reliable;
-                            f->need_env_token = need_env_token;
-                            f->payRequired = payRequired;
-                            f->url = string_dup_measure(url, NULL);
-
-                            f->quitting = KNSManagerGetQuitting(self);
-
-                            *file = &f->dad;
-                        }
+                    static int sReliable = eUninitialized;
+                    if (sReliable == eUninitialized) {
+                        const char * e = getenv("NCBI_VDB_RELIABLE");
+                        if (e == NULL)
+                            sReliable = eNotSet;
+                        else if (e[0] == '\0')
+                            sReliable = eUnreliable;
+                        else
+                            sReliable = eReliable;
                     }
+                    if (sReliable == eUnreliable)
+                        reliable = false;
+                    else if (sReliable == eReliable)
+                        reliable = true;
+
+                    f->conn = conn;
+
+                    f->vers = vers;
+                    f->reliable = reliable;
+                    f->need_env_token = need_env_token;
+                    f->payRequired = payRequired;
+                    f->url = string_dup_measure(url, NULL);
+
+                    f->quitting = KNSManagerGetQuitting(self);
+
+                    /* readWaitMillis and totalReadWaitMillis
+                       ARE NEEDED BY HttpFileGetReadTimeouts() */
+                    f->totalReadWaitMillis
+                        = CalculateTotalReadWait(f, &f->readWaitMillis);
+
+                    f->totalConnectWaitMillis 
+                        = self->maxTotalConnectWaitForReliableURLs_ms;
                 }
             }
+
+            if (rc == 0) {
+                rc = RetrierReopenRemote(f, true);
+                if (rc == 0)
+                    *file = &f->dad;
+            }
+
+            if (rc != 0)
+                KStblHttpFileDestroy(f);
         }
     }
 
@@ -547,12 +682,15 @@ LIB_EXPORT rc_t CC KNSManagerMakeReliableHttpFile(const KNSManager *self,
     va_list args;
     va_start(args, url);
     rc = KNSManagerVMakeHttpFileInt(self, file, conn,
-        vers, true, need_env_token, payRequired, url, args);
+        vers, reliable, need_env_token, payRequired, url, args);
     va_end(args);
     return rc;
 }
 
-LIB_EXPORT bool CC KFileIsKHttpFile(const struct KFile * self)
+LIB_EXPORT bool CC KFileIsKHttpFile(const KFile * self)
 {
-    return self != NULL && &self->vt->v1 == &vtKHttpFile;
+    if (self != NULL && &self->vt->v1 == &vtKHttpFile)
+        return true;
+    else
+        return KUnstableFileIsKHttpFile(self);
 }
