@@ -1,8 +1,3 @@
-/* TODO: move them to interfaces/klib/strings.h */
-#define MAGIC_CACHE_VDBCACHE  "VDB_CACHE_VDBCACHE"
-#define MAGIC_LOCAL_VDBCACHE  "VDB_LOCAL_VDBCACHE"
-#define MAGIC_REMOTE_VDBCACHE "VDB_REMOTE_VDBCACHE"
-
 /*===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -49,14 +44,16 @@
 #include <vfs/manager.h>
 #include <vfs/resolver.h>
 #include <vfs/path.h>
+
 #include <klib/debug.h> /* DBGMSG */
-#include <klib/printf.h>
-#include <klib/rc.h>
-#include <klib/namelist.h>
 #include <klib/log.h>
+#include <klib/namelist.h>
+#include <klib/printf.h>
+#include <klib/strings.h> /* ENV_MAGIC_REMOTE_VDBCACHE */
+#include <klib/rc.h>
+
 #include <sysalloc.h>
 
-#include "../kdb/kdb-priv.h" /* KDBManagerCheckAd */
 #include "../vfs/path-priv.h"     /* VPath */
 #include "../vfs/resolver-priv.h" /* rcResolver */
 
@@ -283,7 +280,7 @@ rc_t VDBManagerVPathOpenLocalDBRead ( const VDBManager *self,
                 return 0;
             }
         }
-        
+
         VDatabaseWhack ( db );
     }
 
@@ -315,7 +312,7 @@ rc_t VDBManagerVPathOpenRemoteDBRead ( const VDBManager *self,
                 return 0;
             }
         }
-        
+
         VDatabaseWhack ( db );
     }
 
@@ -330,7 +327,7 @@ typedef enum {
 typedef enum {
     eCheckFilePathFalse,
     eCheckFilePathTrue,
-} ECheckFilePath; 
+} ECheckFilePath;
 
 typedef enum {
     eCheckUrlFalse,
@@ -374,6 +371,8 @@ static rc_t VFSManagerMagicResolve(const VFSManager *self,
     rc = VFSManagerMakePath(self, path, "%s", magic);
 
     if (rc == 0) {
+        bool high_reliability = true;
+
         assert(path);
 
         if (checkUrl == eCheckUrlTrue) {
@@ -392,14 +391,26 @@ static rc_t VFSManagerMagicResolve(const VFSManager *self,
             }
         }
 
+        if (rc == 0) {
+            const char * e = getenv("NCBI_VDB_RELIABLE");
+            if (e != NULL && e[0] == '\0')
+                high_reliability = false;
+            if (high_reliability)
+                rc = VPathMarkHighReliability((VPath*)*path, true);
+        }
+
         if (rc != 0) {
             VPathRelease(*path);
             *path = NULL;
             return rc;
         }
         else
-            DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_PATH), (
-                "'%s' magic '%s' found\n", name, magic));
+            if (high_reliability)
+                DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_PATH), (
+                    "'%s' reliable magic '%s' found\n", name, magic));
+            else
+                DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_PATH), (
+                    "'%s' unreliable magic '%s' found\n", name, magic));
     }
 
     else
@@ -414,7 +425,7 @@ static rc_t VFSManagerCacheMagicResolve(
     const VFSManager * self, VPath ** path, bool * envVarWasSet)
 {
     return VFSManagerMagicResolve(self, path,
-        MAGIC_CACHE_VDBCACHE,
+        ENV_MAGIC_CACHE_VDBCACHE,
         eCheckExistFalse, eCheckFilePathTrue, eCheckUrlFalse, envVarWasSet);
 }
 
@@ -422,7 +433,7 @@ static rc_t VFSManagerLocalMagicResolve(
     const VFSManager * self, VPath ** path, bool * envVarWasSet)
 {
     return VFSManagerMagicResolve(self, path,
-        MAGIC_LOCAL_VDBCACHE,
+        ENV_MAGIC_LOCAL_VDBCACHE,
         eCheckExistTrue, eCheckFilePathTrue, eCheckUrlFalse, envVarWasSet);
 }
 
@@ -430,7 +441,7 @@ static rc_t VFSManagerRemoteMagicResolve(
     const VFSManager * self, VPath ** path, bool * envVarWasSet)
 {
     return VFSManagerMagicResolve(self, path,
-        MAGIC_REMOTE_VDBCACHE,
+        ENV_MAGIC_REMOTE_VDBCACHE,
         eCheckExistFalse, eCheckFilePathFalse, eCheckUrlTrue, envVarWasSet);
 }
 
@@ -438,7 +449,8 @@ static rc_t DBManagerOpenVdbcache(const VDBManager *self,
     const VFSManager * vfs, const VDatabase * db, const VSchema *schema,
     const VPath * orig, bool is_accession, const VResolver * resolver,
     const VPath ** pcache,
-    const VPath * plocal, const VPath * premote)
+    const VPath * plocal,
+    const VPath ** premote)
     /* just one of plocal | premote is set */
 {
     rc_t rc2 = 0;
@@ -474,7 +486,6 @@ static rc_t DBManagerOpenVdbcache(const VDBManager *self,
                     rc2 = VFSManagerExtractAccessionOrOID(vfs, &acc, orig);
                     if (rc2 == 0)
                     {
-                        VPathRelease(orig);
                         orig = acc;
                     }
                 }
@@ -488,7 +499,7 @@ static rc_t DBManagerOpenVdbcache(const VDBManager *self,
                      */
                     KLogLevel lvl = KLogLevelGet();
                     KLogLevelSet(klogFatal);
-                    assert(premote == NULL);
+                    assert(*premote == NULL);
                     assert(pcache && *pcache == NULL);
      /* YES,
       DBG_VFS should be used here to be printed along with other VFS messages */
@@ -496,29 +507,30 @@ static rc_t DBManagerOpenVdbcache(const VDBManager *self,
                 (">>>>>>>>>>>>>> calling VResolverQuery to locate vdbcache..."
                             "\n"));
                     rc2 = VResolverQuery(
-                        resolver, 0, orig, NULL, &premote, pcache);
+                        resolver, 0, orig, NULL, premote, pcache);
                     assert((rc2 == 0) ||
-                        (rc2 != 0 && premote == NULL));
+                        (rc2 != 0 && *premote == NULL));
 
                     /* Here we are restoring log level
                      */
                     KLogLevelSet(lvl);
+                    VPathRelease(orig);
                 }
             }
         }
     }
 
     /* if principal was remote, or attempting remote vdbcache */
-    if (premote != NULL)
+    if (*premote != NULL)
     {
         rc2 = VFSManagerRemoteMagicResolve(vfs, &cremote, &magicWasSet);
         if (rc2 == 0 && cremote == NULL) {
             /* check if names service returned vdbcache */
             bool vdbcacheChecked = false;
 
-            /* check remote path to vdbcache 
+            /* check remote path to vdbcache
                just when magic variable is not set */
-            rc2 = VPathGetVdbcache(premote,
+            rc2 = VPathGetVdbcache(*premote,
                 (const VPath **)& cremote, &vdbcacheChecked);
 
             /* try to manually build remote vdbcache path
@@ -526,7 +538,7 @@ static rc_t DBManagerOpenVdbcache(const VDBManager *self,
                should never happen these days */
             if (rc2 != 0 || vdbcacheChecked == false)
                 rc2 = VFSManagerMakePathWithExtension(
-                    vfs, (VPath**)& cremote, premote, ".vdbcache");
+                    vfs, (VPath**)& cremote, *premote, ".vdbcache");
         }
 
         if (rc2 == 0) {
@@ -555,9 +567,9 @@ static rc_t DBManagerOpenVdbcache(const VDBManager *self,
     return rc2;
 }
 
-LIB_EXPORT rc_t CC VDBManagerOpenDBReadVPath ( const VDBManager *self,
+static rc_t VDBManagerOpenDBReadVPathImpl ( const VDBManager *self,
     const VDatabase ** dbp, const VSchema *schema,
-    const VPath * path )
+    const VPath * path, bool openVdbcache )
 {
     rc_t rc = 0;
 
@@ -581,7 +593,6 @@ LIB_EXPORT rc_t CC VDBManagerOpenDBReadVPath ( const VDBManager *self,
                 {
                     /* turn spec in "path_fmt" + "args" into VPath "orig" */
                     const VPath * aOrig = path;
-                  /*rc = VFSManagerVMakePath ( vfs, &aOrig, path_fmt, args );*/
                     if ( rc == 0 )
                     {
                         /* the original VPath may get resolved into other paths */
@@ -590,7 +601,7 @@ LIB_EXPORT rc_t CC VDBManagerOpenDBReadVPath ( const VDBManager *self,
                         const VPath * orig = aOrig;
                         bool is_accession;
 
-                        KDBManagerCheckAd(self->kmgr, aOrig, &orig);
+                        VFSManagerCheckEnvAndAd(vfs, aOrig, &orig);
 
                         /* check whether we were given a path or accession */
                         is_accession = VPathIsAccessionOrOID ( orig );
@@ -649,96 +660,33 @@ LIB_EXPORT rc_t CC VDBManagerOpenDBReadVPath ( const VDBManager *self,
                                 const VDatabase * db = * dbp;
                                 if ( VDatabaseIsCSRA ( db ) )
                                 {
-                                    DBManagerOpenVdbcache(self, vfs, db,
-                                        schema, orig, is_accession, resolver, 
-                                        &pcache, plocal, premote);
-#if 0
-                                    /* CSRA databases may have an associated "vdbcache" */
-                                    const VDatabase * vdbcache = NULL;
-                                    VPath * clocal = NULL, * ccache = NULL;
-                                    const VPath * cremote = NULL;
-                                    /* if principal was local */
-                                    if ( plocal != NULL )
-                                    {
-                                        rc2 = VFSManagerMakePathWithExtension ( vfs, & clocal, plocal, ".vdbcache" );
-                                        if ( rc2 == 0 )
-                                        {
-                                            rc2 = VDBManagerVPathOpenLocalDBRead ( self, & vdbcache, schema, clocal );
-                                            if ( rc2 != 0 )
-                                            {
-                                                rc2 = 0;
-                                                if ( ! is_accession )
-                                                {
-                                                    VPath * acc;
-                                                    rc2 = VFSManagerExtractAccessionOrOID ( vfs, & acc, orig );
-                                                    if ( rc2 == 0 )
-                                                    {
-                                                        VPathRelease ( orig );
-                                                        orig = acc;
-                                                    }
-                                                }
-
-                                                /* was not found locally - try to get one remotely */
-                                                if ( rc2 == 0 )
-                                                {
-                                                        /* We need suppress error message in the 
-                                                         * case if here any error happened
-                                                         */
-                                                    KLogLevel lvl = KLogLevelGet ();
-                                                    KLogLevelSet ( klogFatal );
-                                                    assert ( premote == NULL );
-                                                    assert ( pcache == NULL );
-                                                    rc2 = VResolverQuery ( resolver, 0, orig, NULL, & premote, & pcache );
-                                                    assert ( ( rc2 == 0 ) ||
-                                                        ( rc2 != 0 && premote == NULL ) );
-
-                                                        /* Here we are restoring log level
-                                                         */
-                                                    KLogLevelSet ( lvl );
-                                                }
-                                            }
-                                        }
+                                    if ( openVdbcache ) {
+                                        DBGMSG(DBG_VFS,
+                                            DBG_FLAG(DBG_VFS_SERVICE),
+                                            ("VDatabase is CSRA: "
+                                                "searching vdbcache...\n"));
+                                        DBManagerOpenVdbcache(self, vfs, db,
+                                            schema, orig, is_accession,
+                                            resolver,
+                                            &pcache, plocal, &premote);
                                     }
-
-                                    /* if principal was remote, or attempting remote vdbcache */
-                                    if ( premote != NULL )
-                                    {
-                                        /* check
-                                           if names service returned vdbache */
-                                        bool vdbcacheChecked = false;
-                                        rc2 = VPathGetVdbcache ( premote,
-                                            & cremote, & vdbcacheChecked );
-
-                                        /* try to manually build remote vdbcache path
-                                           just when names service was not able to return vdbcache: shold never happen these days */
-                                        if ( rc2 != 0 || vdbcacheChecked == false )
-                                            rc2 = VFSManagerMakePathWithExtension ( vfs, (VPath**) & cremote, premote, ".vdbcache" );
-
-                                        if ( rc2 == 0 && pcache != NULL )
-                                            rc2 = VFSManagerMakePathWithExtension ( vfs, & ccache, pcache, ".vdbcache" );
-                                        if ( rc2 == 0 )
-                                            rc2 = VDBManagerVPathOpenRemoteDBRead ( self, & vdbcache, schema, cremote, ccache );
+                                    else {
+                                        DBGMSG(DBG_VFS,
+                                            DBG_FLAG(DBG_VFS_SERVICE),
+                                            ("VDatabase is CSRA: "
+                                                "skip searching vdbcache\n"));
                                     }
-
-                                    VPathRelease ( clocal );
-                                    VPathRelease ( cremote );
-                                    VPathRelease ( ccache );
-
-                                    /* if "vdbcache" is anything but NULL, we got the cache */
-                                    ( ( VDatabase* ) db ) -> cache_db = vdbcache;
-#endif
                                 }
+                                else
+                                    DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_SERVICE),
+                                        ("VDatabase is not CSRA: "
+                                            "don't use vdbcache\n"));
                             }
 
                             VPathRelease ( plocal );
                             VPathRelease ( premote );
                             VPathRelease ( pcache );
                         }
-
-                        /*if (aOrig != orig)
-                            VPathRelease(aOrig);
-
-                        VPathRelease ( orig );*/
                     }
 
                     VResolverRelease ( resolver );
@@ -756,6 +704,20 @@ LIB_EXPORT rc_t CC VDBManagerOpenDBReadVPath ( const VDBManager *self,
 }
 
 
+LIB_EXPORT rc_t CC VDBManagerOpenDBReadVPath ( const VDBManager *self,
+    const VDatabase ** dbp, const VSchema *schema,
+    const VPath * path )
+{
+    return VDBManagerOpenDBReadVPathImpl ( self, dbp, schema, path, true );
+}
+
+LIB_EXPORT rc_t CC VDBManagerOpenDBReadVPathLight ( const VDBManager *self,
+    const VDatabase ** dbp, const VSchema *schema,
+    const VPath * path )
+{
+    return VDBManagerOpenDBReadVPathImpl ( self, dbp, schema, path, false );
+}
+
 LIB_EXPORT rc_t CC VDBManagerVOpenDBRead ( const VDBManager *self,
     const VDatabase ** dbp, const VSchema *schema,
     const char * path_fmt, va_list args )
@@ -853,7 +815,7 @@ LIB_EXPORT rc_t CC VDBManagerVOpenDBRead ( const VDBManager *self,
                                 if ( VDatabaseIsCSRA ( db ) )
                                 {
                                     DBManagerOpenVdbcache(self, vfs, db,
-                                        schema, orig, is_accession, resolver, 
+                                        schema, orig, is_accession, resolver,
                                         &pcache, plocal, premote);
 #if 0
                                     /* CSRA databases may have an associated "vdbcache" */
@@ -884,7 +846,7 @@ LIB_EXPORT rc_t CC VDBManagerVOpenDBRead ( const VDBManager *self,
                                                 /* was not found locally - try to get one remotely */
                                                 if ( rc2 == 0 )
                                                 {
-                                                        /* We need suppress error message in the 
+                                                        /* We need suppress error message in the
                                                          * case if here any error happened
                                                          */
                                                     KLogLevel lvl = KLogLevelGet ();
@@ -1047,7 +1009,7 @@ LIB_EXPORT bool CC VDatabaseLocked ( const VDatabase *self )
  *  valid values are kptDatabase, kptTable and kptIndex
  *
  *  "resolved" [ OUT ] and "rsize" [ IN ] - optional output buffer
- *  for fundamenta object name if "alias" is not a fundamental name, 
+ *  for fundamenta object name if "alias" is not a fundamental name,
  *
  *  "name" [ IN ] - NUL terminated object name
  */
@@ -1073,7 +1035,7 @@ LIB_EXPORT bool CC VDatabaseIsAlias ( const VDatabase *self, uint32_t type,
  *
  *  "path" [ IN ] - NUL terminated path
  */
-LIB_EXPORT rc_t CC VDatabaseVWritable ( const VDatabase *self, uint32_t type, 
+LIB_EXPORT rc_t CC VDatabaseVWritable ( const VDatabase *self, uint32_t type,
         const char *name, va_list args )
 {
     rc_t rc;
@@ -1225,8 +1187,8 @@ LIB_EXPORT rc_t CC VDatabaseOpenSchema ( const VDatabase *self, const VSchema **
 
 
 /* lists the tables of the database
- * 
- * "names" [ OUT ] - return parameter for tables 
+ *
+ * "names" [ OUT ] - return parameter for tables
  */
 LIB_EXPORT rc_t CC VDatabaseListTbl ( const VDatabase *self, KNamelist **names )
 {
@@ -1253,8 +1215,8 @@ LIB_EXPORT rc_t CC VDatabaseListTbl ( const VDatabase *self, KNamelist **names )
 }
 
 /* lists the sub-databases of the database
- * 
- * "names" [ OUT ] - return parameter for databases 
+ *
+ * "names" [ OUT ] - return parameter for databases
  */
 LIB_EXPORT rc_t CC VDatabaseListDB ( const VDatabase *self, KNamelist **names )
 {
@@ -1406,9 +1368,10 @@ LIB_EXPORT bool CC VDatabaseIsCSRA ( const VDatabase *self )
 static bool validName(const String * acc, const String * file) {
     assert(acc && file);
     const char ext[] = ".sra";
+    const char next[] = ".noqual.sra";
     if (file->size == acc->size + 4) {
         if (string_cmp(file->addr, file->size,
-            acc->addr, acc->size, acc->size) == 0)
+            acc->addr, acc->size, acc->len) == 0)
         {
             if (string_cmp(file->addr + acc->size, file->size - acc->size,
                 ext, sizeof ext - 1, sizeof ext - 1) == 0)
@@ -1418,15 +1381,27 @@ static bool validName(const String * acc, const String * file) {
         }
         return false;
     }
+    else if (file->size == acc->size + 4 + 7) {
+        if (string_cmp(file->addr, file->size,
+            acc->addr, acc->size, acc->len) == 0)
+        {
+            if (string_cmp(file->addr + acc->size, file->size - acc->size,
+                next, sizeof next - 1, sizeof next - 1) == 0)
+            {
+                return true;
+            }
+        }
+        return false;
+    }
     else {
         if (string_cmp(file->addr, file->size,
-            acc->addr, acc->size, acc->size) == 0)
+            acc->addr, acc->size, acc->len) == 0)
         {
             const char sfx[] = "_dbGaP-";
             if (string_cmp(file->addr + acc->size, sizeof sfx - 1,
                 sfx, sizeof sfx - 1, sizeof sfx - 1) == 0)
             {
-                int i = 0;
+                size_t i = 0;
                 for (i = acc->size + sizeof sfx - 1;
                     i < file->size; ++i)
                 {
@@ -1443,6 +1418,11 @@ static bool validName(const String * acc, const String * file) {
                 {
                     return true;
                 }
+                if (string_cmp(file->addr + i, file->size - acc->size,
+                    next, sizeof next - 1, sizeof next - 1) == 0)
+                {
+                    return true;
+                }
             }
         }
         return false;
@@ -1511,3 +1491,39 @@ rc_t CC VDatabaseGetAccession(const VDatabase * self, const String ** aAcc)
 
     return rc;
 }
+
+
+/* GetQualityCapability
+ *  can the database deliver full quality? synthetic quallity?
+ */
+LIB_EXPORT rc_t CC VDatabaseGetQualityCapability ( const VDatabase *self,
+    bool *fullQuality, bool *synthQuality )
+{   /* function stub */
+    if ( self == NULL )
+        return RC ( rcVDB, rcDatabase, rcListing, rcSelf, rcNull );
+    if ( fullQuality != NULL )
+        *fullQuality = true;
+    if ( synthQuality != NULL )
+        *synthQuality = false;
+    return 0;
+}
+
+/* SetFullQualityType
+ *  switch database to deliver full quality
+ */
+LIB_EXPORT rc_t CC VDatabaseSetFullQualityType ( VDatabase *self )
+{   /* function stub */
+    if ( self == NULL )
+        return RC ( rcVDB, rcDatabase, rcResetting, rcSelf, rcNull );
+    return 0;
+}
+
+/* SetSynthQualityType
+ *  switch database to deliver synthetic quality
+ */
+LIB_EXPORT rc_t CC VDatabaseSetSynthQualityType ( VDatabase *self )
+{   /* function stub */
+    if (self == NULL)
+        return RC ( rcVDB, rcDatabase, rcResetting, rcSelf, rcNull );
+    return RC ( rcVDB, rcDatabase, rcResetting, rcMode, rcNotAvailable );
+}
