@@ -20,9 +20,7 @@
 *
 *  Please cite the author in any work or product based on this material.
 *
-* ===========================================================================
-*
-*/
+* =========================================================================== */
 
 #include <klib/debug.h> /* DBGMSG */
 #include <klib/json.h> /* KJsonObject */
@@ -38,6 +36,7 @@
 
 #include "path-priv.h" /* VPathMake */
 #include "resolver-priv.h" /* DEFAULT_PROTOCOLS */
+#include "services-cache.h" /* ServicesCacheWhack */
 
 #include <ctype.h> /* isdigit */
 
@@ -60,6 +59,7 @@ typedef struct File {
     char * name;
 
     int64_t size;
+    VQuality quality;
 
     const VPath * http; /* http path from path[]
 when all path[] are alternative ways to get the same acc by different protocols:
@@ -104,7 +104,7 @@ struct Container {
 };
 
 struct Response4 { /* Response object */
-    atomic32_t refcount; 
+    atomic32_t refcount;
     Status status;
     Container * items;
     uint32_t nItems;
@@ -112,6 +112,9 @@ struct Response4 { /* Response object */
     rc_t rc;
 
     bool dontLogNamesServiceErrors;
+    int64_t projectId; /* when -1: not set */
+
+    ServicesCache * cache;
 };
 
 struct KSrvRespObj {
@@ -554,7 +557,7 @@ rc_t ItemAddFormat ( Item * self, const char * cType, const Data * dad,
             type = eSFFMax;
     }
     if ( self -> elm == NULL )  {
-        size_t n = 1;
+        uint32_t n = 1;
         switch ( type ) {
             case eSFFSkipped : idx = 0; n = 1; break;
             case eSFFSra     : idx = 0; n = 1; break;
@@ -566,6 +569,7 @@ rc_t ItemAddFormat ( Item * self, const char * cType, const Data * dad,
         if ( self -> elm == NULL )
             return RC ( rcVFS, rcQuery, rcExecuting, rcMemory, rcExhausted );
         self->elm->size = -1; /* unknown */
+        self->elm->quality = dad == NULL ? eQualLast : dad->quality;
         self -> nElm = n;
     }
     else {
@@ -677,104 +681,6 @@ rc_t ItemSetTicket ( Item * self, const String * tic ) {
     return 0;
 }
 
-/*rc_t ItemGetId ( const Item * self, const char ** id ) {
-    assert ( self && id );
-
-    * id = self -> acc;
-
-    return 0;
-}
-
-static rc_t ItemAdd ( Item * self, const VPath * path,
-               EType type, const char * cType)
-{
-    int idx = -1;
-    File * elm = NULL;
-    if ( self == NULL )
-        return RC ( rcVFS, rcQuery, rcExecuting, rcSelf, rcNull );
-    if ( path == NULL )
-        return 0;
-    if ( type >= eMax )
-        type = eInvalid;
-    if ( cType == NULL ) {
-        if ( type == eInvalid )
-            return RC ( rcVFS, rcQuery, rcExecuting, rcParam, rcNull );
-    }
-    else if ( type == eInvalid ) {
-        if      ( strcmp ( cType, "sra"      ) == 0 )
-            type = eSra;
-        else if ( strcmp ( cType, "vdbcache" ) == 0 )
-            type = eVdbcache;
-        else
-            type = eMax;
-    }
-    if ( self -> elm == NULL )  {
-        size_t n = 2;
-        switch ( type ) {
-            case eSra     : idx = 0; n = 2; break;
-            case eVdbcache: idx = 1; n = 2; break;
-            case eMax     : idx = 2; n = 3; break;
-            default       :  assert ( 0 );
-        }
-        self -> elm = ( File * ) calloc ( n, sizeof * self -> elm );
-        if ( self -> elm == NULL )
-            return RC ( rcVFS, rcQuery, rcExecuting, rcMemory, rcExhausted );
-        self -> nElm = n;
-    }
-    else {
-        switch ( type ) {
-            case eSra     : idx = 0; break;
-            case eVdbcache: idx = 1; break;
-            case eMax     : {
-                uint32_t i =0 ;
-                for ( i = 2; i < self -> nElm; ++ i ) {
-                    assert ( cType && self -> elm [ i ] . cType );
-                    if ( strcmp ( self -> elm [ i ] . cType, cType ) == 0 ) {
-                        idx = i;
-                        break;
-                    }
-                }
-                if ( idx == -1 ) {
-                    void * tmp = realloc ( self -> elm,
-                        ( self -> nElm + 1 ) * sizeof * self -> elm );
-                    if ( tmp == NULL )
-                        return RC ( rcVFS, rcQuery, rcExecuting,
-                                    rcMemory, rcExhausted );
-                    self -> elm = ( File * ) tmp;
-                    idx = self -> nElm ++;
-                    elm = & self -> elm [ idx ];
-                    memset ( elm, 0, sizeof * elm );
-                }
-                break;
-            }
-            default       :  assert ( 0 );
-        }
-    }
-    assert ( idx >= 0 );
-    elm = & self -> elm [ idx ];
-    if ( elm -> cType == NULL ) {
-        elm -> cType = strdup ( cType );
-        if ( elm -> cType == NULL )
-            return RC ( rcVFS, rcQuery, rcExecuting, rcMemory, rcExhausted );
-        elm -> type = type;
-    }
-    return FileAddVCache ( & self -> elm [ idx ], path );
-}
-
-static rc_t ItemAddVPath ( Item * self, const VPath * path ) {
-    if ( self == NULL )
-        return RC ( rcVFS, rcQuery, rcExecuting, rcSelf, rcNull );
-
-    return ItemAdd ( self, path, eSra, "sra" );
-}
-
-static rc_t ItemAddVbdcache ( Item * self, const VPath * path ) {
-    if ( self == NULL )
-        return RC ( rcVFS, rcQuery, rcExecuting, rcSelf, rcNull );
-
-    return ItemAdd ( self, path, eVdbcache, "vdbcache" );
-}*/
-
 /******************************** Status **************************************/
 
 rc_t StatusInit(Status * self, int64_t code, const char * msg) {
@@ -1043,7 +949,7 @@ rc_t ContainerAddId ( Container * self, uint32_t id, Item ** newItem )
 /********************************* Response4 **********************************/
 
 rc_t Response4Fini ( Response4 * self ) {
-    rc_t rc = 0;
+    rc_t rc = 0, r2 = 0;
 
     uint32_t i = 0;
 
@@ -1059,7 +965,13 @@ rc_t Response4Fini ( Response4 * self ) {
     StatusFini(&self->status);
 
     free ( self -> items );
-    free(self->nextToken);
+    free ( self->nextToken );
+
+    /* not ref-counted */
+    r2 = ServicesCacheWhack(self->cache);
+    if (r2 != 0 && rc == 0)
+        rc = r2;
+    self->cache = NULL;
 
     memset ( self, 0, sizeof * self );
 
@@ -1068,7 +980,7 @@ rc_t Response4Fini ( Response4 * self ) {
 
 rc_t Response4Release ( const Response4 * cself ) {
     rc_t rc = 0;
-    
+
     Response4 * self = ( Response4 * ) cself;
 
     if ( self == NULL )
@@ -1170,23 +1082,19 @@ rc_t Response4AddAccOrId ( Response4 * self, const char * acc,
     return 0;
 }
 
-rc_t Response4AppendUrl ( Response4 * self, const char * url ) {
+rc_t Response4AppendUrlPath ( Response4 * self, const char * acc,
+    const char * url, const VPath * path )
+{
     rc_t rc = 0;
 
-    VPath * path = NULL;
-
     Container * box = NULL;
     Item * item = NULL;
     File * l = NULL;
 
-    rc = VPathMake ( & path, url );
-    if ( rc != 0 )
-        return rc;
-
-    rc = Response4AddAccOrId ( self, url, -1, & box );
+    rc = Response4AddAccOrId ( self, acc, -1, & box );
 
     if ( rc == 0 )
-        rc = ContainerAdd ( box, url, -1, & item, NULL );
+        rc = ContainerAdd ( box, acc, -1, & item, NULL );
 
     if ( rc == 0 )
         rc = ItemAddFormat ( item, "", NULL, & l, true );
@@ -1194,41 +1102,75 @@ rc_t Response4AppendUrl ( Response4 * self, const char * url ) {
     if ( rc == 0 )
         rc = FileAddVPath ( l, path, NULL, false, 0 );
 
+    return rc;
+}
+
+rc_t Response4AppendUrl ( Response4 * self, const char * url ) {
+    VPath * path = NULL;
+
+    rc_t rc = VFSManagerMakePath((VFSManager *)1, & path, "%s", url);
+    if ( rc != 0 )
+        return rc;
+
+    rc = Response4AppendUrlPath ( self, url, url, path );
+
     RELEASE ( VPath, path );
 
     return rc;
 }
 
-/*rc_t Response4AppendFile( Response4 * self, const char * acc,
-                          struct Item ** file )
+rc_t Response4AppendLocalAndCache(Response4 * self,
+    const char * acc, const VPathSet * vps, const VFSManager * mgr)
 {
-    rc_t rc = 0;
+    rc_t rc = 0, aRc = 0;
     Container * box = NULL;
-    rc = Response4AddAccOrId ( self, acc, -1, & box );
-    if ( rc == 0 )
-        rc = ContainerAdd ( box, acc, -1, file );
+    Item * item = NULL;
+    File * l = NULL;
+
+    const VPath * cache = NULL;
+    const VPath * local = NULL;
+
+    assert(self);
+
+    rc = VPathSetGetLocal(vps, &local);
+    if (rc != 0) {
+        if (!self->dontLogNamesServiceErrors)
+            PLOGERR(klogErr, (klogErr,
+                rc, "failed to resolve accession '$(acc)'", "acc=%s", acc));
+        return rc;
+    }
+
+    aRc = VPathSetGetCache(vps, &cache);
+
+    if (rc == 0)
+        rc = Response4AddAccOrId(self, acc, -1, &box);
+
+    if (rc == 0)
+        rc = ContainerAdd(box, acc, -1, &item, NULL);
+
+    if (rc == 0)
+        rc = ItemAddFormat(item, "", NULL, &l, true);
+
+    if (rc == 0)
+        rc = FileAddLocal(l, local, 0);
+
+    if (rc == 0)
+        rc = FileAddCache(l, cache, aRc);
+
+    RELEASE(VPath, cache);
+    RELEASE(VPath, local);
+
     return rc;
 }
 
-
-static rc_t Response4AddAcc
-    ( Response4 * self, const char * acc, Container ** newItem )
+rc_t Response4GetServiceCache(const Response4 * self,
+    ServicesCache ** cache)
 {
-    if ( acc == NULL )
-        return RC ( rcVFS, rcQuery, rcExecuting, rcParam, rcNull );;
-
-    return Response4AddAccOrId ( self, acc, 0, newItem );
+    assert(self && cache);
+    *cache = self->cache;
+    return 0;
 }
 
-static rc_t Response4AddId
-    ( Response4 * self, uint32_t id, Container ** newItem )
-{
-    if ( id == 0 )
-        return RC ( rcVFS, rcQuery, rcExecuting, rcParam, rcNull );
-
-    return Response4AddAccOrId ( self, NULL, id, newItem );
-}*/
-
 /******************************** Data setters ********************************/
 
 rc_t IntSet ( int64_t * self, const KJsonValue * node,
@@ -1473,7 +1415,7 @@ static rc_t FileAddLink ( File * self, const KJsonValue * node,
         const String * objectType = NULL;
         rc = VPathMakeFromUrl ( & path, & url, NULL, true, & acc, dad -> sz,
             dad -> mod, hasMd5 ? md5 : NULL, 0, dad -> srv, objectType, NULL,
-            false, false, NULL, -1, 0 );
+            false, false, NULL, -1, 0, NULL );
     }
     else {
         const String * objectType = NULL;
@@ -1481,7 +1423,7 @@ static rc_t FileAddLink ( File * self, const KJsonValue * node,
         StringInitCString ( & ticket, dad -> tic );
         rc = VPathMakeFromUrl ( & path, & url, & ticket, true, & acc, dad -> sz,
             dad -> mod, hasMd5 ? md5 : NULL, 0, dad -> srv, objectType, NULL,
-            false, false, NULL, -1, 0 );
+            false, false, NULL, -1, 0, NULL );
     }
 
     if ( rc == 0 )
@@ -1631,7 +1573,7 @@ static rc_t ItemMappingByAcc(const Item * self) {
     const char vdbcache[] = "vdbcache";
     const char pileup[] = "pileup";
     const char realign[] = "realign";
-    
+
     assert( self );
 
     if (self->acc != NULL) {
@@ -2020,7 +1962,7 @@ static rc_t Response4AddItems4 ( Response4 * self, Container * aBox,
 
     value = KJsonObjectGetMember ( node, "link" );
 
-    if ( ( ( data . cls != NULL ) && ( strcmp ( data .cls, "run"  ) == 0 
+    if ( ( ( data . cls != NULL ) && ( strcmp ( data .cls, "run"  ) == 0
                                     || strcmp ( data .cls, "file" ) == 0 ) )
         || value != NULL )
     {
@@ -2178,7 +2120,12 @@ static rc_t Response4Init4 ( Response4 * self, const char * input ) {
     return rc;
 }
 
-rc_t Response4MakeEmpty (Response4 ** self, bool logNamesServiceErrors) {
+rc_t Response4MakeEmpty (Response4 ** self, const VFSManager * vfs,
+    const struct KNSManager * kns, const struct KConfig * kfg,
+    bool logNamesServiceErrors, int64_t projectId, unsigned quality)
+{
+    rc_t rc = 0;
+
     const char * env = NULL;
 
     assert ( self );
@@ -2187,7 +2134,15 @@ rc_t Response4MakeEmpty (Response4 ** self, bool logNamesServiceErrors) {
     if ( * self == NULL )
         return RC ( rcVFS, rcQuery, rcExecuting, rcMemory, rcExhausted );
 
-    (*self)->dontLogNamesServiceErrors = ! logNamesServiceErrors;
+    (*self)->dontLogNamesServiceErrors = !logNamesServiceErrors;
+    (*self)->projectId = projectId;
+
+    rc = ServicesCacheMake(&(*self)->cache, vfs, kns, kfg, projectId, quality);
+    if (rc != 0) {
+        free(*self);
+        *self = NULL;
+        return rc;
+    }
 
     env = getenv("NCBI_VDB_JSON");
 
@@ -2219,7 +2174,7 @@ rc_t Response4Make4 ( Response4 ** self, const char * input ) {
 
     assert ( self );
 
-    rc = Response4MakeEmpty ( & r, true );
+    rc = Response4MakeEmpty ( & r, NULL, NULL, NULL, true, -1, 0 );
     if ( rc != 0 )
         return rc;
 
@@ -2232,46 +2187,6 @@ rc_t Response4Make4 ( Response4 ** self, const char * input ) {
     return rc;
 }
 
-/*rc_t JResponseMake ( JResponse ** self, const char * input ) {
-    rc_t rc = 0;
-
-    JResponse * r = ( JResponse * ) calloc ( 1, sizeof * self );
-    if ( r == NULL )
-        return RC ( rcVFS, rcQuery, rcExecuting, rcMemory, rcExhausted );
-
-    assert ( self );
-
-    rc = Response4Init4 ( & r -> r, input );
-    if ( rc != 0 )
-        free ( r );
-    else {
-        atomic32_set ( & r -> refcount, 1 );
-        * self = r;
-    }
-
-    return rc;
-}
-
-rc_t JResponseRelease ( const JResponse * cself ) {
-    rc_t rc = 0;
-
-    JResponse * self = ( JResponse * ) cself;
-
-    if ( self == NULL )
-        return 0;
-
-    if ( ! atomic32_dec_and_test ( & self -> refcount ) )
-        return 0;
-
-    rc = Response4Fini ( & self -> r );
-
-    memset ( self, 0, sizeof * self );
-
-    free ( self );
-
-    return rc;
-}*/
-
 rc_t Response4AddRef ( const Response4 * self ) {
     if ( self != NULL )
         atomic32_inc ( & ( ( Response4 * ) self ) -> refcount );
@@ -2305,7 +2220,15 @@ rc_t Response4GetRc ( const Response4 * self, rc_t * rc ) {
     return 0;
 }
 
-rc_t Response4GetNextToken(const Response4 * self, const char ** nextToken) {
+int64_t Response4GetProjectId(const Response4 * self) {
+    assert(self);
+
+    return self->projectId;
+}
+
+rc_t Response4GetNextToken(const Response4 * self,
+    const char ** nextToken)
+{
     const char * dummy = NULL;
     if (nextToken == NULL)
         nextToken = &dummy;
@@ -2610,7 +2533,28 @@ rc_t KSrvRespFileGetFormat ( const KSrvRespFile * self,
     return 0;
 }
 
-rc_t KSrvRespFileGetClass(const KSrvRespFile * self, const char ** itemClass) {
+rc_t KSrvRespFileGetQuality(const KSrvRespFile * self, VQuality * quality)
+{
+    assert(self && self->file && quality);
+
+    *quality = self->file->quality;
+
+    return 0;
+}
+
+rc_t KSrvRespFileGetAccession(const KSrvRespFile * self,
+    const char ** acc)
+{
+    assert(self && self->item && acc);
+
+    *acc = self->item->acc;
+
+    return 0;
+}
+
+rc_t KSrvRespFileGetClass(const KSrvRespFile * self,
+    const char ** itemClass)
+{
     assert(self && self->item && itemClass);
 
     *itemClass = self->item->itemClass;
@@ -2653,7 +2597,7 @@ static
 rc_t KSrvRespFileGetAccNoTic ( const KSrvRespFile * self, const char ** acc )
 {
     assert ( self && self -> item && acc );
-    
+
     *acc = self->item->acc;
 
     if (self->item->id <= 0) {
@@ -2698,7 +2642,7 @@ rc_t KSrvRespFileGetId ( const KSrvRespFile * self, uint64_t * id,
 
     return 0;
 }
-                                                           
+
 rc_t KSrvRespFileGetHttp ( const KSrvRespFile * self,
                             const VPath ** path )
 {
@@ -2752,7 +2696,10 @@ rc_t KSrvRespFileGetLocal ( const KSrvRespFile * self,
 
     * path = NULL;
 
-    assert ( self && self -> file );
+    if (self == NULL)
+        return RC(rcVFS, rcQuery, rcExecuting, rcSelf, rcNull);
+
+    assert ( self -> file );
 
     if ( self -> file -> localRc != 0 )
         return self -> file -> localRc;
