@@ -44,6 +44,7 @@
 
 #include "path-priv.h" /* EVPathInitError */
 #include "resolver-priv.h" /* VResolverResolveName */
+#include "services-cache.h" /* ServicesCacheWhack */
 #include "services-priv.h" /* KServiceGetResolver */
 
 #include <ctype.h> /* isdigit */
@@ -83,7 +84,7 @@ static int64_t CC BSTItemCmp ( const void * item, const BSTNode * n ) {
     assert ( s && i );
  
     return string_cmp ( s -> addr, s -> size,
-        i -> ticket -> addr, i -> ticket -> size, s -> size );
+        i -> ticket -> addr, i -> ticket -> size, s -> len );
 }
 
 static
@@ -100,18 +101,21 @@ int64_t CC BSTreeSort ( const BSTNode * item, const BSTNode * n )
 typedef struct {
     KService * service; /* DO NOT RELEASE */
     const VFSManager * mgr;
+    ServicesCache * cache;
     const KConfig * kfg;
     VResolver * resolver;
     BSTree ticketsToResolvers;
 } H;
 
-static rc_t HInit ( H * self, KService * s ) {
+static rc_t HInit ( H * self, KService * s, ServicesCache * cache ) {
     rc_t rc = 0;
 
     assert ( self && s );
 
     memset ( self, 0, sizeof * self );
 
+    self -> cache = cache;
+
     self -> service = s;
 
     if ( rc == 0 )
@@ -279,8 +283,8 @@ static rc_t _VFSManagerVPathMakeAndTest ( const VFSManager * self,
     return rc;
 }
 
-static rc_t VPathCacheLocationForSource(
-    const VPath * self, const KDirectory * dir, VPath ** path, rc_t *rcOut)
+static rc_t VPathCacheLocationForSource(const VPath * self,
+    const KDirectory * dir, VPath ** path, rc_t *rcOut)
 {
     rc_t rc = 0;
 
@@ -337,10 +341,11 @@ static rc_t VPathCacheLocationForSource(
     return rc;
 }
 
-static rc_t VResolversQuery ( const VResolver * self, const VFSManager * mgr,
+static rc_t VResolversQuery ( const VResolver * self,
+    const VFSManager * mgr, ServicesCache * servicesCache,
     VRemoteProtocols protocols, const VPath * path, const String * acc,
-    uint64_t id, VPathSet ** result, ESrvFileFormat ff,
-    const char * outDir, const char * outFile, const VPath * mapping )
+    uint64_t id, VPathSet ** result, ESrvFileFormat ff, const char * outDir,
+    const char * outFile, const VPath * mapping, const char * origAcc )
 {
     rc_t rc = 0;
 
@@ -349,7 +354,7 @@ static rc_t VResolversQuery ( const VResolver * self, const VFSManager * mgr,
     uint32_t oid = 0;
     uint32_t i = 0;
 
-    assert ( self && result );
+    assert ( self && result && acc );
 
     if ( id == 0 )
         for ( i = 0; i < acc -> size; ++i ) {
@@ -364,16 +369,31 @@ static rc_t VResolversQuery ( const VResolver * self, const VFSManager * mgr,
         oid = id;
 
     if ( oid == 0 ) {
-        rc = VFSManagerMakePath ( mgr, & query, "%S", acc );
-        if (rc == 0 && path != NULL && path->projectId >= 0) {
-            assert(query);
-            query->projectId = path->projectId;
+        bool vdbcache = false;
+        if (ff == eSFFVdbcache && servicesCache == NULL) {
+            String s;
+            CONST_STRING(&s, ".vdbcache");
+            if (acc->len <= s.len 
+                || strstr(acc->addr, s.addr) == NULL)
+            {
+                vdbcache = true;
+            }
         }
+        if (vdbcache)
+            rc = VFSManagerMakePath ( mgr, & query, "%S.vdbcache", acc );
+        else
+            rc = VFSManagerMakePath ( mgr, & query, "%S", acc );
     }
     else
         rc = VFSManagerMakeOidPath ( mgr, & query, oid );
+    if (rc == 0 && path != NULL && path->projectId >= 0) {
+        assert(query);
+        query->projectId = path->projectId;
+    }
 
     if ( rc == 0 ) {
+        VResolverAppID app = appUnknown;
+
         bool isSource = false;
 
         const VPath * local = NULL;
@@ -388,9 +408,14 @@ static rc_t VResolversQuery ( const VResolver * self, const VFSManager * mgr,
         CONST_STRING(&srapub_files, "srapub_files");
         CONST_STRING(&remote      , "remote"      );
 
-        isSource = StringEqual(&path->objectType, &srapub_files);
-        if ( ! isSource )
-            isSource = StringEqual(&path->objectType, &remote);
+        if (path != NULL) {
+            isSource = StringEqual(&path->objectType, &srapub_files);
+            if (!isSource)
+                isSource = StringEqual(&path->objectType, &remote);
+        }
+
+        app = get_accession_app(acc, false,
+            NULL, NULL, false, NULL, NULL, -1);
 
         if ( outFile != NULL ) {
             bool exists = false;
@@ -408,6 +433,21 @@ static rc_t VResolversQuery ( const VResolver * self, const VFSManager * mgr,
                                       rcName, rcNotFound );
         }
 
+        else if (!VResolverIsProtected(self)
+            && app == appSRA && origAcc != NULL && origAcc[0] == 'S'
+            && servicesCache != NULL)
+        {
+            rc = ServicesCacheResolve(servicesCache, path, &local, &cache);
+            if (rc == 0) {
+                if (local == NULL)
+                    localRc = RC(rcVFS, rcResolver, rcResolving,
+                        rcName, rcNotFound);
+                if (cache == NULL)
+                    cacheRc = RC(rcVFS, rcResolver, rcResolving,
+                        rcName, rcNotFound);
+            }
+        }
+
         else if ( VPathFromUri ( path ) ) {
             if (isSource) {
                 KDirectory * dir = NULL;
@@ -473,9 +513,9 @@ static rc_t VResolversQuery ( const VResolver * self, const VFSManager * mgr,
         }
 
         else {
-            cacheRc = VResolverQuery ( self, protocols, query,
+            cacheRc = VResolverQueryForCache ( self, protocols, query,
                                     NULL, NULL, & cache );
-            localRc = VResolverQuery ( self, protocols, query,
+            localRc = VResolverQueryForCache ( self, protocols, query,
                                     & local, NULL, NULL );
         }
 
@@ -537,6 +577,8 @@ static rc_t _VPathGetId ( const VPath * self, const String ** newId,
     String http;
     String https;
 
+    bool replace = true;
+
     assert ( newId && oldId );
 
     * newId = NULL;
@@ -551,11 +593,11 @@ static rc_t _VPathGetId ( const VPath * self, const String ** newId,
     if ( oldId -> size <= https . size )
         return 0;
 
-    /* what is being attempted with this code? */
+    /* what is being attempted with this code?
     if ( string_cmp ( oldId -> addr, oldId -> size, https . addr, https. size, https. size ) != 0 &&
          string_cmp ( oldId -> addr, oldId -> size, fasp  . addr, fasp . size, fasp . size ) != 0 &&
          string_cmp ( oldId -> addr, oldId -> size, http  . addr, fasp . size, fasp . size ) != 0 )
-    {   return 0; }
+    {   return 0; } */
 
     rc = VPathGetId ( self, & id );
     if ( rc == 0 && id . size == 0 ) {
@@ -599,19 +641,30 @@ static rc_t _VPathGetId ( const VPath * self, const String ** newId,
 
             free ( ( void * ) str );
         }
+
+        if ( rc == 0 )
+            rc = VPathGetPath ( acc_or_oid, & id );
     }
 
-    if ( rc == 0 )
-        rc = VPathGetPath ( acc_or_oid, & id );
-
     if ( rc == 0 ) {
-        _StringFixSrrWithVersion ( & id );
-        rc = StringCopy ( newId, & id );
+        /* return ID just when it's different from oldId
+        (replace file name by numeric [dbGaP] id) */
+        if (oldId != NULL &&
+            oldId->addr != NULL && oldId->size > 0 &&
+            id.addr != NULL && id.size > 0 &&
+            oldId->addr[0] == id.addr[0])
+        {
+            replace = false;
+        }
+        else {
+            _StringFixSrrWithVersion(&id);
+            rc = StringCopy(newId, &id);
+        }
     }
 
     RELEASE ( VPath, acc_or_oid );
 
-    if ( rc == 0 )
+    if ( rc == 0 && replace )
         * oldId = * * newId;
 
     return rc;
@@ -656,62 +709,221 @@ static rc_t KSrvResponseRegisterLocalAndCache(KSrvResponse * self,
     return rc;
 }
 
+static rc_t KServiceResolvers(const KService * self, VRemoteProtocols protocols,
+    const char * outDir, const char * outFile,
+    H * h, VPathSet ** vps, const VPath * path,
+    ESrvFileFormat ff, const char * tic, uint64_t iid, const VPath * mapping,
+    String * id, const char * origAcc)
+{
+    rc_t rc = 0;
+    VResolver * resolver = NULL;
+    const String * pId = NULL;
+    String acc;
+    String ticket;
+    memset(&acc, 0, sizeof acc);
+    memset(&ticket, 0, sizeof ticket);
+    assert(h);
+    if (rc == 0 && tic != NULL)
+        StringInitCString(&ticket, tic);
+    if (rc == 0)
+        rc = HResolver(h, self, &ticket, &resolver, path);
+    if (rc == 0 && path != NULL) {
+        rc = VPathGetAccession(path, &acc);
+        if (rc == 0 && acc.size > 0) {
+            if (!(id->size > 0 && id->addr != NULL && id->addr[0] == 'S' &&
+                acc.size > 0 && acc.addr != NULL && isdigit(acc.addr[0])))
+                /* Don't replace id with acc
+                   when id starts with S but acc is number.
+                   E.g., it happens when id is SRRnnn.pileup */
+            {
+                id = &acc;
+            }
+        }
+        else
+            rc = _VPathGetId(path, &pId, id, h->mgr);
+    }
+    if (rc == 0) {
+        assert(resolver);
+        VResolverResolveName(resolver, KServiceGetResolveName(self));
+        rc = VResolversQuery(resolver, h->mgr, h->cache, protocols, path,
+            id, iid, vps, ff, outDir, outFile, mapping, origAcc);
+    }
+    free((void *)pId);
+    return rc;
+}
+
+#ifdef DBGNG
+#define STS_FIN  3
+#endif
+
 static
 rc_t KServiceNamesQueryExtImpl ( KService * self, VRemoteProtocols protocols, 
     const char * cgi, const char * version, const KSrvResponse ** aResponse,
     const char * outDir, const char * outFile, const char * expected )
 {
-    rc_t rc = 0;
+    rc_t RC_NOT_FND
+        = SILENT_RC(rcVFS, rcQuery, rcResolving, rcName, rcNotFound);
+    rc_t rc = 0, rcc = 0;
+    ServicesCache * cache = NULL;
+    uint32_t i = 0;
+    const KSrvRespObj * obj = NULL;
+    const VPath * path = NULL;
     KSrvResponse * response = NULL;
+    rc_t rx = 0;
+    KSrvRespObjIterator * it = NULL;
+    KSrvRespFile * file = NULL;
+    KSrvRespFileIterator * fi = NULL;
+
+    int32_t q = -1;
+    bool servicesCacheDisabled = true;
+
+#ifdef DBGNG
+    STSMSG(STS_FIN, ("%s: entered", __func__));
+#endif
     if ( aResponse == NULL )
         return RC ( rcVFS, rcQuery, rcExecuting, rcParam, rcNull );
     * aResponse = NULL;
 
-    {
+    {   /* call External Services */
         const KSrvResponse * r = NULL;
+#ifdef DBGNG
+        STSMSG(STS_FIN, ("%s: entering KServiceNamesExecuteExtImpl...",
+            __func__));
+#endif
         rc = KServiceNamesExecuteExtImpl ( self, protocols, cgi,
                                        version, & r, expected );
+#ifdef DBGNG
+        STSMSG(STS_FIN, ("%s: ...KServiceNamesExecuteExtImpl done with %R",
+            __func__, rc));
+#endif
         if ( rc == 0 )
             response = ( KSrvResponse* ) r;
     }
-    
+
+    KServiceGetQuality(self, &q);
+    servicesCacheDisabled = q >= eQualLast || q < 0;
+
+    if (!servicesCacheDisabled) {
+#ifdef DBGNG
+        STSMSG(STS_FIN, ("%s: calling KServiceGetServiceCache...", __func__));
+#endif
+        if ((rc == 0 || rc == RC_NOT_FND) && KServiceCallsSdl(self))
+            rcc = KServiceGetServiceCache(self, &cache);
+
+        if (rc == 0 && KServiceCallsSdl(self)) {
+            /* add each file from External Services result to cache */
+            uint32_t n = KSrvResponseLength(response);
+#ifdef DBGNG
+            STSMSG(STS_FIN, ("%s: calling ServicesCacheAddRemote...", __func__));
+#endif
+            for (i = 0; rc == 0 && i < n; ++i) {
+                rc = KSrvResponseGetObjByIdx(response, i, &obj);
+                if (rc == 0) {
+                    rc = KSrvRespObjGetError(obj, &rx, NULL, NULL);
+                    if (rc == 0 && rx == 0) {
+                        rc = KSrvRespObjMakeIterator(obj, &it);
+                        while (rc == 0) {
+                            rc = KSrvRespObjIteratorNextFile(it, &file);
+                            if (rc != 0 || file == NULL)
+                                break;
+                            else {
+                                const char * acc = NULL;
+                                rc = KSrvRespFileGetAccOrId(file, &acc, NULL);
+                                if (rc == 0
+                                    && acc != NULL && acc[0] != '\0'
+                                    && acc[1] == 'R' && acc[2] == 'R')
+                                {
+                                    rc = KSrvRespFileMakeIterator(file, &fi);
+                                    if (rc == 0) {
+                                        rc = KSrvRespFileIteratorNextPath(fi,
+                                            &path);
+                                        if (rc == 0)
+                                            rc = ServicesCacheAddRemote(cache,
+                                                path);
+                                        RELEASE(VPath, path);
+                                    }
+                                    RELEASE(KSrvRespFileIterator, fi);
+                                }
+                                RELEASE(KSrvRespFile, file);
+                            }
+                        }
+                        RELEASE(KSrvRespObjIterator, it);
+                    }
+                }
+                RELEASE(KSrvRespObj, obj);
+            }
+        }
+
+        if (rcc == 0 && KServiceCallsSdl(self)) {
+#ifdef DBGNG
+            STSMSG(STS_FIN, ("%s: before calling ServicesCacheComplete...",
+                __func__));
+#endif
+            if (rc == RC_NOT_FND) {
+                uint32_t i = 0;
+                for (i = 0; ; ++i) {
+                    const char * acc = KServiceGetId(self, i);
+                    if (acc == NULL)
+                        break;
+                    rcc = ServicesCacheAddId(cache, acc);
+                }
+            }
+            if (rcc == 0 && (rc == 0 || rc == RC_NOT_FND)) {
+                bool skipLocal = KServiceSkipLocal(self);
+#ifdef DBGNG
+                STSMSG(STS_FIN, ("%s: entering ServicesCacheComplete...",
+                    __func__));
+#endif
+                rcc = ServicesCacheComplete(cache, outDir, outFile, skipLocal);
+#ifdef DBGNG
+                STSMSG(STS_FIN, ("%s: ...ServicesCacheComplete done with %R",
+                    __func__, rcc));
+#endif
+                if (rcc != 0 && rc == 0)
+                    rc = rcc;
+            }
+        }
+    }
+
     if ( rc == 0 ) {
         H h;
-        rc = HInit ( & h, self );
+        rc = HInit ( & h, self, cache );
+
+#ifdef DBGNG
+        STSMSG(STS_FIN, ("%s: iterating  KSrvResponse...", __func__));
+#endif
 
         if ( protocols == eProtocolDefault )
              protocols = DEFAULT_PROTOCOLS;
         
         {
-            uint32_t i = 0;
             uint32_t n = KSrvResponseLength  ( response );
             for ( i = 0; rc == 0 && i < n; ++ i ) {
-                const KSrvRespObj * obj = NULL;
                 VPathSet * vps = NULL;
-                const VPath * path = NULL;
                 const KSrvError * error = NULL;
                 rc = KSrvResponseGetObjByIdx ( response, i, & obj );
                 if ( rc == 0 ) {
-                    rc_t rx = 0;
                     rc = KSrvRespObjGetError(obj, &rx, NULL, NULL);
                     if (rc == 0 && rx == 0) {
-                        KSrvRespObjIterator * it = NULL;
                         rc = KSrvRespObjMakeIterator(obj, &it);
                         while (rc == 0) {
-                            KSrvRespFile * file = NULL;
                             rc = KSrvRespObjIteratorNextFile(it, &file);
                             if (rc != 0 || file == NULL)
                                 break;
                             else {
                                 ESrvFileFormat ff = eSFFInvalid;
-                                KSrvRespFileIterator * fi = NULL;
                                 const char * acc = NULL;
                                 const char * tic = NULL;
                                 uint64_t iid = 0;
                                 const VPath * mapping = NULL;
                                 String id;
+                                const char * origAcc = NULL;
                                 memset(&id, 0, sizeof id);
-                                rc = KSrvRespFileGetAccOrName(file, &acc, &tic);
+                                rc = KSrvRespFileGetAccOrId(file,
+                                    &origAcc, NULL);
+                                if (rc == 0)
+                                    rc = KSrvRespFileGetAccOrName(file,
+                                        &acc, &tic);
                                 if (rc == 0) {
                                     if (acc != NULL) {
                                         StringInitCString(&id, acc);
@@ -732,33 +944,11 @@ rc_t KServiceNamesQueryExtImpl ( KService * self, VRemoteProtocols protocols,
                                     rc = KSrvRespFileIteratorNextPath(
                                         fi, &path);
                                     if (rc == 0) {
-                                        if (error == NULL) {
-                                            VResolver * resolver = NULL;
-                                            const String * pId = NULL;
-                                            String ticket;
-                                            memset(&ticket,
-                                                0, sizeof ticket);
-                                            if (rc == 0 && tic != NULL)
-                                                StringInitCString(&ticket,
-                                                    tic);
-                                            if (rc == 0)
-                                                rc = HResolver(&h, self,
-                                                    &ticket, &resolver, path);
-                                            if (rc == 0)
-                                                rc = _VPathGetId(path, &pId,
-                                                    &id, h.mgr);
-                                            if (rc == 0) {
-                                                assert(resolver);
-                                                VResolverResolveName(resolver,
-                                                    KServiceGetResolveName(
-                                                        self));
-                                                rc = VResolversQuery(resolver,
-                                                    h.mgr, protocols, path,
-                                                    &id, iid, &vps, ff,
-                                                    outDir, outFile, mapping);
-                                            }
-                                            free((void *)pId);
-                                        }
+                                        if (error == NULL)
+                                            rc = KServiceResolvers(self,
+                                                protocols, outDir, outFile,
+                                                &h, &vps, path, ff, tic, iid,
+                                                mapping, &id, origAcc);
                                         else
                                             RELEASE(KSrvError, error);
                                     }
@@ -801,8 +991,8 @@ rc_t KServiceNamesQueryExtImpl ( KService * self, VRemoteProtocols protocols,
                                 VResolverResolveName ( resolver,
                                             KServiceGetResolveName ( self ) );
                                 rc = VResolversQuery ( resolver, h . mgr,
-                                    protocols, path, & id, 0, & vps,
-                                    eSFFInvalid, outDir, outFile, NULL );
+                                    h . cache, protocols, path, & id, 0, & vps,
+                                    eSFFInvalid, outDir, outFile, NULL, NULL );
                             }
                             RELEASE ( VPath, acc_or_oid );
                         }
@@ -825,7 +1015,56 @@ rc_t KServiceNamesQueryExtImpl ( KService * self, VRemoteProtocols protocols,
                 rc = r2;
         }
     }
+    else if (rc == RC_NOT_FND) {
+        if (KServiceGetId(self, 0) == NULL) /* request is empty */
+            return rc;
+        else {
+            uint32_t i = 0;
+            H h;
+            rc = HInit(&h, self, cache);
 
+#ifdef DBGNG
+            STSMSG(STS_FIN, ("%s: iterating  KServiceGetId...", __func__));
+#endif
+
+            for (i = 0; ; ++i) {
+                VPathSet * vps = NULL;
+                ESrvFileFormat ff = eSFFInvalid;
+                uint64_t iid = 0;
+                String id;
+                const char * acc = KServiceGetId(self, i);
+                if (acc == NULL)
+                    break;
+                StringInitCString(&id, acc);
+                if (rc == 0)
+                    rc = KServiceResolvers(self, protocols, outDir, outFile,
+                        &h, &vps, NULL, ff, NULL, iid, NULL, &id, NULL);
+                if (rc == 0) {
+                    if (rc == 0)
+                        rc = KServiceAddLocalAndCacheToResponse(self, acc, vps);
+                    else
+                        rc = 0;
+                    RELEASE(VPathSet, vps);
+                    if (rc == 0)
+                        rc = KServiceGetResponse(self, aResponse);
+                }
+            }
+            {
+                rc_t r2 = HFini(&h);
+                if (r2 != 0 && rc == 0)
+                    rc = r2;
+            }
+        }
+    }
+
+    /*rx = ServicesCacheWhack(cache);
+    if (rx != 0 && rc == 0)
+        rc = rx;
+    cache = NULL;*/
+
+#ifdef DBGNG
+    STSMSG(STS_FIN, ("%s: exiting with %R", __func__, rc));
+#endif
     return rc;
 }
 
