@@ -63,7 +63,7 @@ public:
     ~CSRA1_Fixture()
     {
     }
-    
+
     virtual void Release()
     {
         if (m_ctx != 0)
@@ -75,8 +75,8 @@ public:
         }
         NGS_C_Fixture :: Release ();
     }
-    
-    
+
+
     NGS_Alignment*      m_align;
 };
 
@@ -113,32 +113,32 @@ const uint64_t GetReadCount_PartiallyAligned = GetReadCount_Aligned - GetReadCou
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReadCount_All, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    REQUIRE_EQ( GetReadCount_Total, NGS_ReadCollectionGetReadCount ( m_coll, ctx, true, true, true ) ); 
+    REQUIRE_EQ( GetReadCount_Total, NGS_ReadCollectionGetReadCount ( m_coll, ctx, true, true, true ) );
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReadCount_Unaligned, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    REQUIRE_EQ( GetReadCount_Unaligned, NGS_ReadCollectionGetReadCount ( m_coll, ctx, false, false, true ) ); 
+    REQUIRE_EQ( GetReadCount_Unaligned, NGS_ReadCollectionGetReadCount ( m_coll, ctx, false, false, true ) );
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReadCount_Aligned, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    REQUIRE_EQ( GetReadCount_Aligned, NGS_ReadCollectionGetReadCount ( m_coll, ctx, true, true, false ) ); 
+    REQUIRE_EQ( GetReadCount_Aligned, NGS_ReadCollectionGetReadCount ( m_coll, ctx, true, true, false ) );
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReadCount_FullyAligned, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    REQUIRE_EQ( GetReadCount_FullyAligned, NGS_ReadCollectionGetReadCount ( m_coll, ctx, true, false, false ) ); 
+    REQUIRE_EQ( GetReadCount_FullyAligned, NGS_ReadCollectionGetReadCount ( m_coll, ctx, true, false, false ) );
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReadCount_PartiallyAligned, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    REQUIRE_EQ( GetReadCount_PartiallyAligned, NGS_ReadCollectionGetReadCount ( m_coll, ctx, false, true, false ) ); 
+    REQUIRE_EQ( GetReadCount_PartiallyAligned, NGS_ReadCollectionGetReadCount ( m_coll, ctx, false, true, false ) );
     EXIT;
 }
 
@@ -146,57 +146,57 @@ FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReadCount_PartiallyAligned, CSRA1_Fixt
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReads_All, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    
+
     m_read = NGS_ReadCollectionGetReads ( m_coll, ctx, true, true, true );
     REQUIRE ( ! FAILED () && m_read );
     REQUIRE ( NGS_ReadIteratorNext ( m_read, ctx ) );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReads_Aligned, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    
+
     m_read = NGS_ReadCollectionGetReads ( m_coll, ctx, true, false, false);
     REQUIRE ( ! FAILED () && m_read );
     REQUIRE ( NGS_ReadIteratorNext ( m_read, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_EQ ( NGS_ReadCategory_fullyAligned, NGS_ReadGetReadCategory ( m_read, ctx ) );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReads_Partial, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    
+
     m_read = NGS_ReadCollectionGetReads ( m_coll, ctx, false, true, false);
     REQUIRE ( ! FAILED () && m_read );
     REQUIRE ( NGS_ReadIteratorNext ( m_read, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_EQ ( NGS_ReadCategory_partiallyAligned, NGS_ReadGetReadCategory ( m_read, ctx ) );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReads_Unaligned, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    
+
     m_read = NGS_ReadCollectionGetReads ( m_coll, ctx, false, false, true);
     REQUIRE ( ! FAILED () && m_read );
     REQUIRE ( NGS_ReadIteratorNext ( m_read, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_EQ ( NGS_ReadCategory_unaligned, NGS_ReadGetReadCategory ( m_read, ctx ) );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetReads_None, CSRA1_Fixture)
 {
     ENTRY_ACC( CSRA1_PrimaryOnly );
-    
+
     m_read = NGS_ReadCollectionGetReads ( m_coll, ctx, false, false, false);
     REQUIRE ( ! FAILED () && m_read );
     REQUIRE ( ! NGS_ReadIteratorNext ( m_read, ctx ) );
-    
+
     EXIT;
 }
 
@@ -250,35 +250,35 @@ FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetReferences, CSRA1_Fixture)
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetReadGroups, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_PrimaryOnly);
-    
+
     m_readGroup = NGS_ReadCollectionGetReadGroups ( m_coll, ctx );
     REQUIRE ( ! FAILED () && m_readGroup );
 
     REQUIRE ( NGS_ReadGroupIteratorNext ( m_readGroup, ctx ) );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetReadGroup_NotFound, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_PrimaryOnly);
     REQUIRE_NULL ( NGS_ReadCollectionGetReadGroup ( m_coll, ctx, "wontfindme" ) );
     REQUIRE_FAILED ();
     EXIT;
 }
-    
+
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetReadGroup_WithGroups_DefaultNotFound, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_PrimaryOnly);
     REQUIRE_NULL ( NGS_ReadCollectionGetReadGroup ( m_coll, ctx, "" ) );
     REQUIRE_FAILED ();
     EXIT;
 }
-    
+
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetReadGroup_WithGroups_Found, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_PrimaryOnly);
     m_readGroup = NGS_ReadCollectionGetReadGroup ( m_coll, ctx, "C1ELY.6");
     REQUIRE_NOT_NULL ( m_readGroup );
@@ -286,70 +286,70 @@ FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetReadGroup_WithGroups_Found, CSRA1_Fixtu
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionHasReadGroup_NotFound, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_PrimaryOnly);
     REQUIRE ( ! NGS_ReadCollectionHasReadGroup ( m_coll, ctx, "wontfindme" ) );
     EXIT;
 }
-    
+
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionHasReadGroup_WithGroups_DefaultNotFound, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_PrimaryOnly);
     REQUIRE ( ! NGS_ReadCollectionHasReadGroup ( m_coll, ctx, "" ) );
     EXIT;
 }
-    
+
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionHasReadGroup_WithGroups_Found, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_PrimaryOnly);
     REQUIRE ( NGS_ReadCollectionHasReadGroup ( m_coll, ctx, "C1ELY.6") );
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetAlignments_All, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_WithSecondary);
-    
+
     m_align = NGS_ReadCollectionGetAlignments ( m_coll, ctx, true, true );
     REQUIRE ( ! FAILED () && m_align );
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE ( NGS_AlignmentIsPrimary ( m_align, ctx ) );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetAlignments_FilteredPrimary, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_WithSecondary);
-    
+
     m_align = NGS_ReadCollectionGetAlignments ( m_coll, ctx, true, false );
     REQUIRE ( ! FAILED () && m_align );
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE ( NGS_AlignmentIsPrimary ( m_align, ctx ) );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetAlignments_FilteredSecondary, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_WithSecondary);
-    
+
     m_align = NGS_ReadCollectionGetAlignments ( m_coll, ctx, false, true );
     REQUIRE ( ! FAILED () && m_align );
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE ( ! NGS_AlignmentIsPrimary ( m_align, ctx ) );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollectionGetAlignments_FilteredNone, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_WithSecondary);
-    
+
     m_align = NGS_ReadCollectionGetAlignments ( m_coll, ctx, false, false );
     REQUIRE ( ! FAILED () && m_align );
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
@@ -400,36 +400,36 @@ FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentCount_Secondary_CategorySecon
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_Primary, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_PrimaryOnly );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 3, 2, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_PrimaryOnly ) + ".PA.3", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_PrimaryOnly ) + ".PA.4", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_PrimarySingle, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_PrimaryOnly );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 4, 1, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_PrimaryOnly ) + ".PA.4", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
@@ -439,7 +439,7 @@ FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_PrimaryOutOfRange_CutLo
 
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 0, 13, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_PrimaryOnly ) + ".PA.1", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
@@ -452,9 +452,9 @@ FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_PrimaryOutOfRangeHigh,
 
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 3987702, 1, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
@@ -464,33 +464,33 @@ FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_PrimaryOutOfRange_CutHi
 
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 3987701, 2, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_PrimaryOnly ) + ".PA.3987701", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_Secondary, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_WithSecondary );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 170, 2, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".SA.170", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".SA.171", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
@@ -498,123 +498,123 @@ FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_Secondary, CSRA1_Fixtur
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_SecondarySingle, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_WithSecondary );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 174, 1, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".SA.174", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_SecondaryOutOfRangeHigh, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_WithSecondary );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 179, 1, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_SecondaryOutOfRange_CutHigh, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_WithSecondary );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 178, 4, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".SA.178", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRange_All, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_WithSecondary );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 167, 4, true, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".PA.167", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".PA.168", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".SA.169", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".SA.170", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRangeFiltered_Primary, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_WithSecondary );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 167, 4, true, false );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".PA.167", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".PA.168", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRangeFiltered_Secondary, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_WithSecondary );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 167, 4, false, true );
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".SA.169", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( NGS_AlignmentIteratorNext ( m_align, ctx ) );
     REQUIRE ( ! FAILED () );
     REQUIRE_STRING ( string ( CSRA1_WithSecondary ) + ".SA.170", NGS_AlignmentGetAlignmentId ( m_align, ctx ) );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_ReadCollection_GetAlignmentRangeFiltered_None, CSRA1_Fixture)
 {
     ENTRY_ACC ( CSRA1_WithSecondary );
-    
+
     m_align = NGS_ReadCollectionGetAlignmentRange ( m_coll, ctx, 167, 4, false, false);
     REQUIRE ( ! FAILED () && m_align );
-    
+
     REQUIRE ( ! NGS_AlignmentIteratorNext ( m_align, ctx ) );
-    
+
     EXIT;
 }
 
@@ -627,12 +627,12 @@ FIXTURE_TEST_CASE(NGS_ReadCollection_MakeAlignmentCursor_Reuse, CSRA1_Fixture)
     REQUIRE ( ! FAILED () && c1 );
     NGS_CursorRelease ( c1, ctx );
     REQUIRE ( ! FAILED () );
-    
+
     // request an exclusive cursor, verify that it is the same one as former c1
     const NGS_Cursor* c2 = CSRA1_ReadCollectionMakeAlignmentCursor ( ( CSRA1_ReadCollection * ) m_coll, ctx, true, true );
     REQUIRE ( ! FAILED () && c2 );
     REQUIRE_EQ ( c1, c2 );
-    
+
     // request another non-exclusive cursor, make sure it is a new one
     c1 = CSRA1_ReadCollectionMakeAlignmentCursor ( ( CSRA1_ReadCollection * ) m_coll, ctx, true, false );
     REQUIRE ( ! FAILED () && c1 );
@@ -641,7 +641,7 @@ FIXTURE_TEST_CASE(NGS_ReadCollection_MakeAlignmentCursor_Reuse, CSRA1_Fixture)
     NGS_CursorRelease ( c2, ctx );
     REQUIRE ( ! FAILED () );
     NGS_CursorRelease ( c1, ctx );
-    
+
     EXIT;
 }
 FIXTURE_TEST_CASE(NGS_ReadCollection_MakeAlignmentCursor_New, CSRA1_Fixture)
@@ -651,33 +651,33 @@ FIXTURE_TEST_CASE(NGS_ReadCollection_MakeAlignmentCursor_New, CSRA1_Fixture)
     // create a non-exclusive cursor
     const NGS_Cursor* c1 = CSRA1_ReadCollectionMakeAlignmentCursor ( ( CSRA1_ReadCollection * ) m_coll, ctx, true, false );
     REQUIRE ( ! FAILED () && c1 );
-    
+
     // request an exclusive cursor, verify that it is not c1
     const NGS_Cursor* c2 = CSRA1_ReadCollectionMakeAlignmentCursor ( ( CSRA1_ReadCollection * ) m_coll, ctx, true, true );
     REQUIRE ( ! FAILED () && c2 );
     REQUIRE_NE ( c1, c2 );
     NGS_CursorRelease ( c2, ctx );
     REQUIRE ( ! FAILED () );
-    
+
     // request a non-exclusive cursor, verify that it is the same as c1
     c2 = CSRA1_ReadCollectionMakeAlignmentCursor ( ( CSRA1_ReadCollection * ) m_coll, ctx, true, false );
     REQUIRE ( ! FAILED () && c2 );
     REQUIRE_EQ ( c1, c2 );
-    
+
     NGS_CursorRelease ( c2, ctx );
     REQUIRE ( ! FAILED () );
     NGS_CursorRelease ( c1, ctx );
-    
+
     EXIT;
 }
 
 FIXTURE_TEST_CASE(CSRA1_NGS_ReadCollectionGetStats, CSRA1_Fixture)
-{   
+{
     ENTRY_ACC(CSRA1_WithSecondary);
-    
+
     NGS_Statistics * stats = NGS_ReadCollectionGetStatistics ( m_coll, ctx );
     REQUIRE ( ! FAILED () );
-    
+
     REQUIRE_EQ ( (uint64_t)615696,  NGS_StatisticsGetAsU64 ( stats, ctx, "SEQUENCE/BASE_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)615696,  NGS_StatisticsGetAsU64 ( stats, ctx, "SEQUENCE/BIO_BASE_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)598728,  NGS_StatisticsGetAsU64 ( stats, ctx, "SEQUENCE/CMP_BASE_COUNT" ) );
@@ -691,21 +691,21 @@ FIXTURE_TEST_CASE(CSRA1_NGS_ReadCollectionGetStats, CSRA1_Fixture)
     REQUIRE_EQ ( (uint64_t)788,     NGS_StatisticsGetAsU64 ( stats, ctx, "REFERENCE/SPOT_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)788,     NGS_StatisticsGetAsU64 ( stats, ctx, "REFERENCE/SPOT_MAX" ) );
     REQUIRE_EQ ( (uint64_t)1,       NGS_StatisticsGetAsU64 ( stats, ctx, "REFERENCE/SPOT_MIN" ) );
-    
+
     REQUIRE_EQ ( (uint64_t)16968,   NGS_StatisticsGetAsU64 ( stats, ctx, "PRIMARY_ALIGNMENT/BASE_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)16968,   NGS_StatisticsGetAsU64 ( stats, ctx, "PRIMARY_ALIGNMENT/BIO_BASE_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)168,     NGS_StatisticsGetAsU64 ( stats, ctx, "PRIMARY_ALIGNMENT/SPOT_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)168,     NGS_StatisticsGetAsU64 ( stats, ctx, "PRIMARY_ALIGNMENT/SPOT_MAX" ) );
     REQUIRE_EQ ( (uint64_t)1,       NGS_StatisticsGetAsU64 ( stats, ctx, "PRIMARY_ALIGNMENT/SPOT_MIN" ) );
-    
+
     REQUIRE_EQ ( (uint64_t)1010,    NGS_StatisticsGetAsU64 ( stats, ctx, "SECONDARY_ALIGNMENT/BASE_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)1010,    NGS_StatisticsGetAsU64 ( stats, ctx, "SECONDARY_ALIGNMENT/BIO_BASE_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)10,      NGS_StatisticsGetAsU64 ( stats, ctx, "SECONDARY_ALIGNMENT/SPOT_COUNT" ) );
     REQUIRE_EQ ( (uint64_t)10,      NGS_StatisticsGetAsU64 ( stats, ctx, "SECONDARY_ALIGNMENT/SPOT_MAX" ) );
     REQUIRE_EQ ( (uint64_t)1,       NGS_StatisticsGetAsU64 ( stats, ctx, "SECONDARY_ALIGNMENT/SPOT_MIN" ) );
-    
+
     NGS_StatisticsRelease ( stats, ctx );
-    
+
     EXIT;
 }
 
@@ -734,8 +734,9 @@ const char UsageDefaultName[] = "test-ngs_csra1_readcollection";
 rc_t CC KMain ( int argc, char *argv [] )
 {
     KConfigDisableUserSettings();
-    rc_t m_coll=NgsCsra1ReadCollectionTestSuite(argc, argv);
-    return m_coll;
+    rc_t ret=NgsCsra1ReadCollectionTestSuite(argc, argv);
+    NGS_C_Fixture::ReleaseCache();
+    return ret;
 }
 
 }
