@@ -240,14 +240,14 @@ public:
 
         // KConfigReadString creates NULL-terminated strings
         REQUIRE ( user_root );
-		std::cout << std::string(user_root->addr) << std::endl;
+//std::cout << std::string(user_root->addr) << std::endl;
         REQUIRE_RC ( KDirectoryRemove ( native, true, "%s", user_root -> addr ) );
 
         VFSManager * vfs = NULL;
 
 //KConfigPrint(cfg, 0);
 
-        REQUIRE_RC ( VFSManagerMakeFromKfg ( & vfs, cfg ) );
+        REQUIRE_RC ( VFSManagerMakeLocal ( & vfs, cfg ) );
         REQUIRE_RC ( SET :: resetSingleton ( vfs ) );
 
         REQUIRE_RC ( SET :: set ( vfs, v ) );
